/*
 * Copyright (C) 2008 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "PlatformMouseEvent.h"

#include "CurrentTime.h"
#include "Assertions.h"
#include "NotImplemented.h"

namespace WKAL {

// FIXME: Would be even better to figure out which modifier is Alt instead of always using GDK_MOD1_MASK.

// Keep this in sync with the other platform event constructors
PlatformMouseEvent::PlatformMouseEvent(BalEventButton* event)
{
    int x = 0, y = 0;
    int modifiers = 0;
    int type = 0;
    int button = 0;

    switch(event->clazz)
    {
        case DFEC_WINDOW:
            {
                DFBWindowEvent* winEvent = (DFBWindowEvent*)event;
                type = winEvent->type;
                modifiers = winEvent->modifiers;
                button = winEvent->button;
                x = winEvent->cx;
                y = winEvent->cy;
            }
            break;
        case DFEC_INPUT:
            {
                DFBInputEvent *inputEvent = (DFBInputEvent*)event;
                type = inputEvent->type;
                modifiers = inputEvent->type;
                button = inputEvent->button;
                x = inputEvent->axisabs;
                y = inputEvent->axisrel; // FIXME
            }
            break;
         default:
            ASSERT(0);
    }

    //m_balEventKey = event;
    m_timestamp = WTF::currentTime();
    m_position = IntPoint(x, y);
    m_globalPosition = m_position;
    m_shiftKey = modifiers & DIMM_SHIFT;
    m_ctrlKey = modifiers & DIMM_CONTROL;
    m_altKey = modifiers & DIMM_ALT;
    m_metaKey = modifiers & DIMM_META;

    m_eventType = MouseEventMoved;
    m_button = NoButton;
    switch(type)
    {
        case DIET_BUTTONPRESS:
        case DWET_BUTTONDOWN:
            m_eventType = MouseEventPressed;
            m_clickCount = 1;
            break;
        case DIET_BUTTONRELEASE:
        case DWET_BUTTONUP:
            m_eventType = MouseEventReleased;
            m_clickCount = 0;
            break;
        case DIET_AXISMOTION:
        case DWET_MOTION:
            break;
        default:
            ASSERT(0);
    }
    if (button == DIBI_LEFT)
        m_button = LeftButton;
    else if(button == DIBI_MIDDLE)
        m_button = MiddleButton;
    else if(button == DIBI_RIGHT)
        m_button = RightButton;
}

} //namespace WKAL

