#ifndef __REF_COUNTED_FONT_FACE_H__
#define __REF_COUNTED_FONT_FACE_H__

#include "BALBase.h"
#include <wtf/RefCounted.h>
#include <wtf/OwnPtr.h>
#include <wtf/PassRefPtr.h>
#include <AtomicString.h>

#if PLATFORM(DFB_CAIRO)
#include <cairo-ft.h>
#endif

namespace WKAL {

#if PLATFORM(DFB)
#if PLATFORM(DFB_CAIRO)
    typedef cairo_scaled_font_t BalNativeFontFace;
#else
    typedef IDirectFBFont BalNativeFontFace;
#endif
#else
    typedef struct FT_FaceRec BalNativeFontFace;
#endif

class RefCountedFontFace : public RefCounted<RefCountedFontFace> {
public:
    static PassRefPtr<RefCountedFontFace> createFontFace(const char* filename, int id, int size);
    static PassRefPtr<RefCountedFontFace> createFontFace(const SharedBuffer* buffer, int id, int size);

    ~RefCountedFontFace();
private:
    RefCountedFontFace(FT_Face face, BalNativeFontFace* nativeFont)
        : m_face(face)
        , m_nativeFont(nativeFont)
    {}
public:
    FT_Face fontFace() { return m_face; }
    BalNativeFontFace* nativeFontFace() { return m_nativeFont;}

private:
    static bool init();
    static FT_Library m_library;
    //static NativeFontFaceCache* m_nativeFontFaceCache;
private:

    FT_Face m_face;
    BalNativeFontFace* m_nativeFont;
};

}
#endif//__NATIVE_FONT_FACE_H__

