/*
 * Copyright (C) 2006 Apple Computer, Inc.
 * Copyright (C) 2008, 2009 Google, Inc.
 *
 * Portions are Copyright (C) 2001 mozilla.org
 *
 * Other contributors:
 *   Stuart Parmenter <stuart@mozilla.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Alternatively, the contents of this file may be used under the terms
 * of either the Mozilla Public License Version 1.1, found at
 * http://www.mozilla.org/MPL/ (the "MPL") or the GNU General Public
 * License Version 2.0, found at http://www.fsf.org/copyleft/gpl.html
 * (the "GPL"), in which case the provisions of the MPL or the GPL are
 * applicable instead of those above.  If you wish to allow use of your
 * version of this file only under the terms of one of those two
 * licenses (the MPL or the GPL) and not to allow others to use your
 * version of this file under the LGPL, indicate your decision by
 * deletingthe provisions above and replace them with the notice and
 * other provisions required by the MPL or the GPL, as the case may be.
 * If you do not delete the provisions above, a recipient may use your
 * version of this file under any of the LGPL, the MPL or the GPL.
 */


#include "NativeImageDFB.h"

#include <cairo-directfb.h>
#include <cstdio>

namespace WebCore {
    
static DFBResult createDFBSurface(const int width, const int height, IDirectFBSurface** pSurface)
{
    IDirectFB* dfb = fl_dfb;
    DFBSurfaceDescription desc;
    DFBResult ret;

    desc.flags = (DFBSurfaceDescriptionFlags)(DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT | DSDESC_CAPS);
    desc.caps = DSCAPS_PREMULTIPLIED;
    desc.width = width;
    desc.height = height;
    desc.pixelformat = DSPF_ARGB;
    ret = dfb->CreateSurface(dfb, &desc, pSurface);

    return ret;
}

bool NativeImageDFB::setSize(const int width, const int height)
{
    createDFBSurface(width, height, &m_surface);
    if (m_surface) {
        m_width = width;
        m_height = height;
        eraseARGB(0, 0, 0, 0);

        DFBResult ret = m_surface->Lock(m_surface,(DFBSurfaceLockFlags)(/*DSLF_READ | */DSLF_WRITE), &m_data, &m_pitch);
        if (ret != DFB_OK) {
            printf(" faile to lock surface %s \n", DirectFBErrorString(ret));
        }
        printf("{%d, %d} m_data = %p, m_pitch = %d\n", width, height, m_data, m_pitch);fflush(stdout);
        ASSERT(m_data);
        ASSERT(m_pitch >= width * 4);
        m_surface->Unlock(m_surface); //FIXME
        return true;
    }
    return false;
}

bool NativeImageDFB::copyTo(NativeImageDFB& other) const
{
    if (!m_surface)
        return false;
    if (!other.setSize(m_width, m_height))
        return false;
    other.m_surface->Blit(other.m_surface, m_surface, NULL, 0, 0);
    other.m_hasAlpha = m_hasAlpha;
    return true;
}

void NativeImageDFB::reset()
{
    if (m_surface) {
        m_width = 0;
        m_height = 0;
        m_surface->Release(m_surface);
        m_surface = NULL;
        m_data = NULL;
        m_pitch = 0;
        m_hasAlpha = false;
    }
}

}


