/*
 * Copyright (C) 2006 Apple Computer, Inc.
 * Copyright (C) 2008, 2009 Google, Inc.
 *
 * Portions are Copyright (C) 2001 mozilla.org
 *
 * Other contributors:
 *   Stuart Parmenter <stuart@mozilla.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Alternatively, the contents of this file may be used under the terms
 * of either the Mozilla Public License Version 1.1, found at
 * http://www.mozilla.org/MPL/ (the "MPL") or the GNU General Public
 * License Version 2.0, found at http://www.fsf.org/copyleft/gpl.html
 * (the "GPL"), in which case the provisions of the MPL or the GPL are
 * applicable instead of those above.  If you wish to allow use of your
 * version of this file only under the terms of one of those two
 * licenses (the MPL or the GPL) and not to allow others to use your
 * version of this file under the LGPL, indicate your decision by
 * deletingthe provisions above and replace them with the notice and
 * other provisions required by the MPL or the GPL, as the case may be.
 * If you do not delete the provisions above, a recipient may use your
 * version of this file under any of the LGPL, the MPL or the GPL.
 */

#ifndef _NATIVE_IMAGE_DFB_
#define _NATIVE_IMAGE_DFB_

#include "config.h"
#include "BALBase.h"

namespace WebCore {
    
    class NativeImageDFB {

    public:
        NativeImageDFB()
            : m_width(0), m_height(0)
            , m_surface(0), m_data(0), m_pitch(0)
            , m_hasAlpha(false)
        {}

        NativeImageDFB(const NativeImageDFB& other)
        {
            *this = other;
        }

        const NativeImageDFB& operator=(const NativeImageDFB& other)
        {
            m_width = other.m_width;
            m_height = other.m_height;
            if (m_surface)
                m_surface->Release(m_surface);
            m_surface = other.m_surface;
            if (m_surface)
                m_surface->AddRef(m_surface);
            m_data = other.m_data;
            m_pitch = other.m_pitch;
            m_hasAlpha = other.m_hasAlpha;
            return *this;
        }

        ~NativeImageDFB()
        {
            reset();
        }

    public:
        bool setSize(const int width, const int height);
        bool copyTo(NativeImageDFB& other) const;
        void reset();
        void eraseARGB(uint8_t a, uint8_t r, uint8_t g, uint8_t b)
        {
            if (m_surface)
                m_surface->Clear(m_surface, r, g, b, a);
            m_hasAlpha = true;
        }
        bool hasAlpha() const { return m_hasAlpha; }
        void setHasAlpha(bool alpha) { m_hasAlpha = alpha; }

        uint32_t* getAddr32(const int x, const int y)
        {
            return (uint32_t*)((char*)m_data + y * m_pitch + (x << 2));
        }
        int pitch() const { return m_pitch; }
        int width() const { return m_width; }
        int height() const { return m_height; }
        IDirectFBSurface* dfbSurface() { return m_surface; }
        const IDirectFBSurface* dfbSurface() const { return m_surface; }
                
    private:
        int m_width;
        int m_height;
        IDirectFBSurface *m_surface;
        void* m_data;
        int m_pitch;
        bool m_hasAlpha;
    };

}

#endif //_NATIVE_IMAGE_DFB_



