/*
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "config.h"
#include "CookieJar.h"

#include "CookieManager.h"
#include "Document.h"
#include "KURL.h"
#include "PlatformString.h"
#include "CString.h"

#define WTF_USE_ORIGYN_COOKIEJAR_IMPL 0

#if USE(ORIGYN_COOKIEJAR_IMPL)
#include "WebCookieJar.h"
#else
#include "StringHash.h"
#include <wtf/HashMap.h>
#endif

namespace WebCore {

void setCookies(Document* document, const KURL& url, const String& value)
{
#if USE(ORIGYN_COOKIEJAR_IMPL)
    WebCookieJar::webCookieJar()->setCookies(std::string(url.string().utf8().data()), std::string(value.utf8().data()));
#else
    const KURL& documentURL = document->url();
    const KURL& cookieURL = documentURL.isEmpty() ? url : documentURL;
 
    cookieManager().setCookies(cookieURL, value);
#endif
}

String cookies(const Document* document, const KURL& url)
{
#if USE(ORIGYN_COOKIEJAR_IMPL)
    std::string c = WebCookieJar::webCookieJar()->cookies(std::string(url.string().utf8().data()));
    return String(c.c_str());
#else
    const KURL& documentURL = document->url();
    const KURL& cookieURL = documentURL.isEmpty() ? url : documentURL;
    
    return cookieManager().getCookie(cookieURL, NoHttpOnlyCookie);
#endif    
}

bool cookiesEnabled(const Document* /*document*/)
{
#if USE(ORIGYN_COOKIEJAR_IMPL)
    return WebCookieJar::webCookieJar()->isEnabled();
#else
    return true;
#endif    
}

}
