#ifndef FL_HostWindowImpl_h
#define FL_HostWindowImpl_h

#include "HostWindow.h"
#include <wtf/RefCounted.h>
#include <wtf/PassRefPtr.h>

#include <FL/Fl_Window.H>
#include <directfb.h>
#include <cairo.h>
#include <cairo-directfb.h>

namespace WebCore {

class FramelessScrollView;

class FL_HostWindowImpl : public Fl_Window
                        , public HostWindow
                        , public RefCounted<FL_HostWindowImpl>
{
public:        
    static PassRefPtr<FL_HostWindowImpl> create(IntRect& widgetRect, FramelessScrollView* widget) { return adoptRef(new FL_HostWindowImpl(widgetRect, widget));}
    ~FL_HostWindowImpl();

private:
    FL_HostWindowImpl(IntRect& widgetRect, FramelessScrollView* widget);

public:
    // Fl_Window methods
    virtual void draw();
    virtual int handle(int);
    virtual void show();
    virtual void hide();
    
public:    
    
    // WebCore::HostWindow methods:
    virtual void repaint(
        const WebCore::IntRect&, bool contentChanged, bool immediate = false,
        bool repaintContentOnly = false);
    virtual void scroll(
        const WebCore::IntSize& scrollDelta, const WebCore::IntRect& scrollRect,
        const WebCore::IntRect& clipRect);
    virtual WebCore::IntPoint screenToWindow(const WebCore::IntPoint&) const;
    virtual WebCore::IntRect windowToScreen(const WebCore::IntRect&) const;
    virtual PlatformWidget platformWindow() const { return 0; }
    virtual void scrollRectIntoView(const WebCore::IntRect&, const WebCore::ScrollView*) const;
    virtual void scrollbarsModeDidChange() const;
    
private:
    FramelessScrollView* m_widget;
    
    cairo_surface_t* m_cairoSurface;
    cairo_t* m_context;
    IDirectFBSurface* m_surface;
    
    IntRect m_dirtyRect;        	    
};

}

#endif

