
#include "AudioPlayer.h"
#include <stdio.h>
#include <acnavi.h>
#include "log.h"
//#include <WebKitATV.h>

#define MAX_ACC_FILE_SIZE 512*1024
#define MAX_WAV_FILE_SIZE 96*1024

CAudioPlayer::CAudioPlayer()
{
	m_file_buff = NULL;
	m_max_file_size = 0;
}

CAudioPlayer::~CAudioPlayer()
{
	if(m_file_buff != NULL){
		delete m_file_buff;
		m_file_buff = NULL;
	}	
}

int CAudioPlayer::NewSource(const char* url, const char* type, int file_size)
{
	LOG("url = %s, type = (%s), file size = (%d).", url, type, file_size);
	
	# if 0
	// url start with "file://", this is for test.
	if(strncmp(url, "file", 4) == 0){
		const char* real_url = url + 7;
		FILE* fp = fopen(real_url,"r");
		if(fp != NULL){
			fseek(fp, 0, SEEK_END);
			m_file_size = ftell(fp);
			fclose(fp);
		}
		else {
			return -1;
		}
	}
	// url start with "http://".
	else {
		m_file_size = file_size;
	}
	#endif

	m_file_offset = 0;
	m_file_size = file_size;

	if(strcasecmp(type,"audio/wav") == 0 || strcasecmp(type,"audio/vnd.wav") == 0 || strcasecmp(type,"audio/x-wav") == 0){
		m_type = ACNAVI_AUDIODATA_PCM;
		m_max_file_size = MAX_WAV_FILE_SIZE;
	}
	else if(strcasecmp(type,"audio/x-aac") == 0){
		m_type = ACNAVI_AUDIODATA_AAC;
		m_max_file_size = MAX_ACC_FILE_SIZE;
	}
	else {
		ERROR("audio type(%s) is unsupport.", type);
		return -1;
	}

	if(m_file_size > m_max_file_size){
		m_file_size = m_max_file_size;
		LOG("resize file size = (%lu) for original file size is too big(%d).", m_file_size, file_size);
	}
	
	if(m_file_buff != NULL){
		delete m_file_buff;
		m_file_buff = NULL;
	}
	m_file_buff = new unsigned char[m_file_size];

	return 0;
}

int CAudioPlayer::WriteData(long offset, long len, const unsigned char* buffer)
{
	LOG("offset (%d), len(%d)", offset, len);
	int real_len = 0;

	// write all the data.
	if(offset + len <= m_file_size){
		real_len = len;
	}
	// write the data witch offset is not bigger than file size.
	else if(offset < m_file_size && offset+len > m_file_size){
		real_len = m_file_size - offset;
	}
	// ignore the rest data.
	else {
		LOG("the file size is ");
		return 0;
	}
	
	memcpy(m_file_buff+offset, (unsigned char*)buffer, len);
	m_file_offset += len;
	if(m_file_offset == m_file_size){
		// auto play.
		Play();
	}
	return real_len;
}

int CAudioPlayer::Play()
{
	LOG("");
	AcNavi_PlayAudioFile((AcNavi_AudioDataType_e)m_type,(void*)m_file_buff, m_file_size);
}


