#include "Plugin.h"
#include "ScriptablePluginObject.h"
#include "log.h"
#include <string>

#ifndef STRINGZ_TO_NPVARIANT_EXT
#define STRINGZ_TO_NPVARIANT_EXT(_val, _v)                                    \
NP_BEGIN_MACRO                                                                \
    (_v).type = NPVariantType_String;                                         \
	size_t _v_len = strlen(_val);											  \
	NPUTF8* pRetBuf = (NPUTF8*)NPN_MemAlloc((uint32)(_v_len) + 1);					  \
	memset(pRetBuf, 0, _v_len + 1);											  \
	memcpy(pRetBuf, _val, _v_len);											  \
    NPString str = { pRetBuf, (uint32_t)_v_len };                                       \
    (_v).value.stringValue = str;                                             \
NP_END_MACRO
#endif



void ScriptablePluginObjectBase::Invalidate()
{
}

bool ScriptablePluginObjectBase::HasMethod(NPIdentifier name)
{
	return false;
}

bool ScriptablePluginObjectBase::Invoke(NPIdentifier name, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return false;
}

bool ScriptablePluginObjectBase::InvokeDefault(const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return false;
}

bool ScriptablePluginObjectBase::HasProperty(NPIdentifier name)
{
	return false;
}

bool ScriptablePluginObjectBase::GetProperty(NPIdentifier name, NPVariant *result)
{
	return false;
}

bool ScriptablePluginObjectBase::SetProperty(NPIdentifier name, const NPVariant *value)
{
	return false;
}

bool ScriptablePluginObjectBase::RemoveProperty(NPIdentifier name)
{
	return false;
}

bool ScriptablePluginObjectBase::Enumerate(NPIdentifier **identifier, uint32_t *count)
{
	return false;
}

bool ScriptablePluginObjectBase::Construct(const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return false;
}

// static
void ScriptablePluginObjectBase::_Deallocate(NPObject *npobj)
{
	// Call the virtual destructor.
	delete (ScriptablePluginObjectBase *)npobj;
}

// static
void ScriptablePluginObjectBase::_Invalidate(NPObject *npobj)
{
	((ScriptablePluginObjectBase *)npobj)->Invalidate();
}

// static
bool ScriptablePluginObjectBase::_HasMethod(NPObject *npobj, NPIdentifier name)
{
	return ((ScriptablePluginObjectBase *)npobj)->HasMethod(name);
}

// static
bool ScriptablePluginObjectBase::_Invoke(NPObject *npobj, NPIdentifier name, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return ((ScriptablePluginObjectBase *)npobj)->Invoke(name, args, argCount, result);
}

// static
bool ScriptablePluginObjectBase::_InvokeDefault(NPObject *npobj, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return ((ScriptablePluginObjectBase *)npobj)->InvokeDefault(args, argCount, result);
}

// static
bool ScriptablePluginObjectBase::_HasProperty(NPObject * npobj, NPIdentifier name)
{
	return ((ScriptablePluginObjectBase *)npobj)->HasProperty(name);
}

// static
bool ScriptablePluginObjectBase::_GetProperty(NPObject *npobj, NPIdentifier name, NPVariant *result)
{
	return ((ScriptablePluginObjectBase *)npobj)->GetProperty(name, result);
}

// static
bool ScriptablePluginObjectBase::_SetProperty(NPObject *npobj, NPIdentifier name, const NPVariant *value)
{
	return ((ScriptablePluginObjectBase *)npobj)->SetProperty(name, value);
}

// static
bool ScriptablePluginObjectBase::_RemoveProperty(NPObject *npobj, NPIdentifier name)
{
	return ((ScriptablePluginObjectBase *)npobj)->RemoveProperty(name);
}

// static
bool ScriptablePluginObjectBase::_Enumerate(NPObject *npobj, NPIdentifier **identifier, uint32_t *count)
{
	return ((ScriptablePluginObjectBase *)npobj)->Enumerate(identifier, count);
}

// static
bool ScriptablePluginObjectBase::_Construct(NPObject *npobj, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return ((ScriptablePluginObjectBase *)npobj)->Construct(args, argCount, result);
}

bool ScriptablePluginObject::HasMethod(NPIdentifier name)
{
	CPlugin* plugin = (CPlugin*)mNpp->pdata;
	if (plugin == NULL)
		return false;
	return plugin->HasMethod(name);
}

bool ScriptablePluginObject::HasProperty(NPIdentifier name)
{
	CPlugin* plugin = (CPlugin*)mNpp->pdata;

	if (plugin == NULL)
		return false;
	return plugin->HasProperty(name);
}

bool ScriptablePluginObject::GetProperty(NPIdentifier name, NPVariant *result)
{
    VOID_TO_NPVARIANT(*result);

    CPlugin* plugin = (CPlugin*)mNpp->pdata;

    if (plugin == NULL)
        return false;
    
    return plugin->GetProperty(name, result);
}

bool ScriptablePluginObject::SetProperty(NPIdentifier name, const NPVariant *value)
{
	CPlugin* plugin = (CPlugin*)mNpp->pdata;

	if (plugin == NULL)
		return false;
    
	return plugin->SetProperty(name, value);
}

bool ScriptablePluginObject::Invoke(NPIdentifier name, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	CPlugin* plugin = (CPlugin*)mNpp->pdata;

	if (plugin == NULL)
		return false;

	return plugin->Invoke(name, args, argCount, result);
}

bool ScriptablePluginObject::InvokeDefault(const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	printf ("ScriptablePluginObject default method called!\n");

	STRINGZ_TO_NPVARIANT_EXT("default method return val", *result);

	return true;
}

