//////////////////////////////////////////////////
//
// CPlugin class implementation
//
#ifdef XP_WIN
#include <windows.h>
#include <windowsx.h>
#endif

#ifdef XP_MAC
#include <TextEdit.h>
#endif

#ifdef XP_UNIX
#endif

#include <string>
#include <string.h>
#include "npupp.h"
#include "Plugin.h"
#include "ScriptablePluginObject.h"
#include "log.h"
#include "SubWindowDisplay.h"

#define STREAM_MAX_BUFF_SIZE 1024*100		// Byte.

extern NPNetscapeFuncs NPNFuncs;

using namespace std;

#define STRINGZ_TO_NPVARIANT_EXT(_val, _v)                          \
	NP_BEGIN_MACRO                                                  \
	(_v).type = NPVariantType_String;                               \
	size_t _v_len = strlen(_val);                                   \
	NPUTF8* pRetBuf = (NPUTF8*)NPN_MemAlloc((uint32)(_v_len) + 1);  \
	memset(pRetBuf, 0, _v_len + 1);                                 \
	memcpy(pRetBuf, _val, _v_len);                                  \
	NPString str = { pRetBuf, (uint32_t)_v_len };                   \
	(_v).value.stringValue = str;                                   \
	NP_END_MACRO

#define INT32_TO_NPVARIANT_EXT(_val, _v)                          \
	NP_BEGIN_MACRO                                                  \
	(_v).type = NPVariantType_Int32;                               \
	(_v).value.intValue = _val;                                   \
	NP_END_MACRO

static NPObject* AllocateScriptablePluginObject(NPP npp, NPClass *aClass)
{
	return new ScriptablePluginObject(npp);
}

DECLARE_NPOBJECT_CLASS_WITH_BASE(ScriptablePluginObject, AllocateScriptablePluginObject);

CPlugin::CPlugin(NPP pNPInstance):  
	m_pNPInstance(pNPInstance),
	m_pNPStream(NULL),
	m_bInitialized(FALSE),
	m_pScriptableObject(NULL)
{
#ifdef XP_WIN
	m_hWnd = NULL;
#endif

	//if(NPN_GetValue(m_pNPInstance, NPNVWindowNPObject, &m_sWindowObj) != NPERR_NO_ERROR)
	//{
	//	LOG("sWindowObj error!");
	//}
	//else
	//{
	//	LOG("sWindowObj ok!");
	//}
	//LOG("window:%p, %p", m_pNPInstance, m_sWindowObj);

	const char *ua = NPN_UserAgent(m_pNPInstance);
	strcpy(m_String, ua);

	m_subWindowDisplay = new CSubWindowDisplay();

	LOG("init plugin success.");
}


CPlugin::~CPlugin()
{
	LOG("destruct plugin...");
	//if (m_sWindowObj)
	//	NPN_ReleaseObject(m_sWindowObj);
	if (m_pScriptableObject)
		NPN_ReleaseObject(m_pScriptableObject);
	//m_sWindowObj = 0;

	delete m_subWindowDisplay;

	LOG("desctuct plugin success.");
}

#ifdef XP_WIN
static LRESULT CALLBACK PluginWinProc(HWND, UINT, WPARAM, LPARAM);
static WNDPROC lpOldProc = NULL;
#endif

NPBool CPlugin::init(NPWindow* pNPWindow)
{
	if(pNPWindow == NULL)
		return FALSE;

#ifdef XP_WIN
	m_hWnd = (HWND)pNPWindow->window;
	if(m_hWnd == NULL)
		return FALSE;

	// subclass window so we can intercept window messages and
	// do our drawing to it
	lpOldProc = SubclassWindow(m_hWnd, (WNDPROC)PluginWinProc);

	// associate window with our CPlugin object so we can access 
	// it in the window procedure
	SetWindowLong(m_hWnd, GWL_USERDATA, (LONG)this);
#endif

	m_Window = pNPWindow;
	m_subWindowDisplay->updateWindow(pNPWindow->x, pNPWindow->y,
	 pNPWindow->width, pNPWindow->height);

	m_bInitialized = TRUE;
	return TRUE;
}

void CPlugin::shut()
{
#ifdef XP_WIN
	// subclass it back
	SubclassWindow(m_hWnd, lpOldProc);
	m_hWnd = NULL;
#endif

	m_bInitialized = FALSE;
}

NPBool CPlugin::isInitialized()
{
	return m_bInitialized;
}

int16 CPlugin::handleEvent(void* event)
{
#ifdef XP_MAC
	NPEvent* ev = (NPEvent*)event;
	if (m_Window) {
		Rect box = { m_Window->y, m_Window->x,
			m_Window->y + m_Window->height, m_Window->x + m_Window->width };
		if (ev->what == updateEvt) {
			::TETextBox(m_String, strlen(m_String), &box, teJustCenter);
		}
	}
#endif
	return 0;
}

// this will force to draw a version string in the plugin window
void CPlugin::showVersion()
{
	const char *ua = NPN_UserAgent(m_pNPInstance);
	strcpy(m_String, ua);

#ifdef XP_WIN
	InvalidateRect(m_hWnd, NULL, TRUE);
	UpdateWindow(m_hWnd);
#endif

	if (m_Window) {
		NPRect r =
		{
			(uint16)m_Window->y, (uint16)m_Window->x,
			(uint16)(m_Window->y + m_Window->height),
			(uint16)(m_Window->x + m_Window->width)
		};

		NPN_InvalidateRect(m_pNPInstance, &r);
	}
}

// this will clean the plugin window
void CPlugin::clear()
{
	strcpy(m_String, "");

#ifdef XP_WIN
	InvalidateRect(m_hWnd, NULL, TRUE);
	UpdateWindow(m_hWnd);
#endif
}

void CPlugin::getVersion(char* *aVersion)
{
	const char *ua = NPN_UserAgent(m_pNPInstance);
	char*& version = *aVersion;
	version = (char*)NPN_MemAlloc(1 + strlen(ua));
	if (version)
		strcpy(version, ua);
}

NPObject* CPlugin::GetScriptableObject()
{
	if (!m_pScriptableObject) {
		m_pScriptableObject = NPN_CreateObject(m_pNPInstance, GET_NPOBJECT_CLASS(ScriptablePluginObject));
	}

	if (m_pScriptableObject) {
		NPN_RetainObject(m_pScriptableObject);
	}

	return m_pScriptableObject;
}

#ifdef XP_WIN
static LRESULT CALLBACK PluginWinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
		case WM_PAINT:
			{
				// draw a frame and display the string
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(hWnd, &ps);
				RECT rc;
				GetClientRect(hWnd, &rc);
				FrameRect(hdc, &rc, GetStockBrush(BLACK_BRUSH));
				CPlugin * p = (CPlugin *)GetWindowLong(hWnd, GWL_USERDATA);
				if(p) {
					if (p->m_String[0] == 0) {
						strcpy("foo", p->m_String);
					}

					DrawTextA(hdc, p->m_String, strlen(p->m_String), &rc, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
				}

				EndPaint(hWnd, &ps);
			}
			break;
		default:
			break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}
#endif

bool CPlugin::HasProperty(NPIdentifier name)
{
	return false;
	char *pName = NPNFuncs.utf8fromidentifier(name);
	if(pName)
	{
		if(strcasecmp((const char *)pName, "src") == 0) {}
		else {
			return false;
		}
		return true;
	}
	return false;
}

bool CPlugin::GetProperty(NPIdentifier name, NPVariant *result)
{
	return false;
}

bool CPlugin::SetProperty(NPIdentifier name, const NPVariant *value)
{
	return false;
	char *pName = NPNFuncs.utf8fromidentifier(name);
	if(pName){
		if(strcasecmp(pName, "src") == 0 && value && value->value.stringValue.UTF8Characters)
		{
			std::string url;
			url.assign(value->value.stringValue.UTF8Characters,value->value.stringValue.UTF8Length);
			LOG("src url = (%s), length = (%d).", url.c_str(), url.length());
			// get URL and next will happen in NewStream.
			NPN_GetURL(m_pNPInstance, url.c_str(), NULL);
			return true;
		}
	}
	return false;
}

bool CPlugin::HasMethod(NPIdentifier methodName)
{
	char *name = NPNFuncs.utf8fromidentifier(methodName);
	if(name){
		if(strcasecmp((const char *)name, "request") == 0){
		}
		else {
			// return false if it is not defined.
			return false;
		}
		return true;
	}
	return false;
}

bool CPlugin::Invoke(NPIdentifier methodName, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	char *name = NPNFuncs.utf8fromidentifier(methodName);

	string str;
	if(argCount > 0 && args[0].type == NPVariantType_String){
		str.assign(args[0].value.stringValue.UTF8Characters,args[0].value.stringValue.UTF8Length);
	}

	int ret = 0;
	if(name){
		LOG("cmd = %s", str.c_str());
		if(strcasecmp((const char *)name, "request") == 0){
			ret = m_subWindowDisplay->request(str.c_str());
		}
	}

	LOG("ret = %d", ret);
	INT32_TO_NPVARIANT_EXT(ret, *result);
	return true;
}

NPError CPlugin::NewStream(NPP instance, NPMIMEType type, NPStream* stream, NPBool seekable, uint16* stype)
{
	return NPERR_GENERIC_ERROR;	
	//return NPERR_NO_ERROR;
}


int32 CPlugin::WriteReady(NPP instance, NPStream* stream)
{
	return 0;
}

int32 CPlugin::Write(NPP instance, NPStream* stream, int32 offset, int32 len, void* buffer)
{
	return -1;
}

NPError CPlugin::DestroyStream(NPP instance, NPStream* stream,	NPReason reason)
{
	LOG("");
	return NPERR_NO_ERROR;
}

