
#include <acnavi.h>
#include <WebKitATV.h>

#include "SubWindowDisplay.h"
#include <string.h>
#include <stdlib.h>
#include "log.h"
#include "Plugin.h"

CSubWindowDisplay::CSubWindowDisplay()
{
	m_x = 0;
	m_y = 0;
	m_w = 0;
	m_h = 0;
}

void CSubWindowDisplay::updateWindow(unsigned long x, unsigned long y, unsigned long w, unsigned long h)
{
	m_x = x;
	m_y = y;
	m_w = w;
	m_h = h;
	LOG("x = %lu, y = %lu, width = %lu, height = %lu.", m_x, m_y, m_w, m_h);
}

static int DownloadCallbackFunc(void* ctx, void* metaData, unsigned long length)
{
    CSubWindowDisplay* player = (CSubWindowDisplay*)ctx;
    if (player && metaData && length > 0) {
        return player->didFinishLoadingCPC(metaData, length);
    }
    return -1;
}

long CSubWindowDisplay::request(const char * cmd)
{
	if(strncasecmp(cmd, "play", 4) == 0){
		return play(cmd+5);
	}
	else if(strcasecmp(cmd, "pause") == 0){
		return pause();
	}
	else if(strcasecmp(cmd, "resume") == 0){
		return resume();
	}
	else if(strncasecmp(cmd, "seek", 4) == 0){
		return seek(cmd + 5);
	}
	else if(strcasecmp(cmd, "stop") == 0){
		return stop();
	}
	else if(strcasecmp(cmd, "status") == 0){
		return status();
	}
	else {
		return -1;
	}
}

long CSubWindowDisplay::play(const char * url)
{
    LOG("%s", url);
    return StartDownloadCPC(url, DownloadCallbackFunc, this); 
}

long CSubWindowDisplay::didFinishLoadingCPC(void* metaData, unsigned long length)
{
    LOG("AcNavi_StartVideo(%p, %d, %d, %d, %d, %d, %d)\n", metaData, length, 0, m_w, m_h, m_x, m_y);
    return AcNavi_StartVideo((char*)metaData, length, 0, m_w, m_h, m_x, m_y);
}

long CSubWindowDisplay::pause()
{
	LOG("");
	return AcNavi_TrickVideo(ACNAVI_TRICKCOMMAND_PAUSE, 0);
}

long CSubWindowDisplay::resume()
{
	LOG("");
	return AcNavi_TrickVideo(ACNAVI_TRICKCOMMAND_RESUM, 0);
}

long CSubWindowDisplay::seek(const char * pos)
{
	LOG("");
	if(strncasecmp(pos,"PlayPos=",8) != 0){
		LOG("");
		return -1;
	}
	pos += 8;
	if(strlen(pos) > 7){
		LOG("");
		return -1;
	}
	int nPos = atoi(pos);
	if(nPos < 0 || nPos > 100000){
		LOG("");
		return -1;
	}
	LOG("seek = %s", nPos);
	// pos 
	return AcNavi_TrickVideo(ACNAVI_TRICKCOMMAND_SEEK, nPos);
}

long CSubWindowDisplay::stop()
{
	LOG("");
	return AcNavi_StopVideo();
}

long CSubWindowDisplay::status()
{
	LOG("");
	int status = 0;
	int ret = 0;
	ret = AcNavi_GetVideoStatus(&status);
	if(ret == ACTV_OK){
		return status;
	}
	return ret;
}

