/*
 * Copyright (C) 2006, 2007 Apple Inc.  All rights reserved.
 * Copyright (C) 2008 Collabora Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "PluginView.h"

#include "Document.h"
#include "DocumentLoader.h"
#include "Element.h"
#include "EventNames.h"
#include "FrameLoader.h"
#include "FrameLoadRequest.h"
#include "FrameTree.h"
#include "Frame.h"
#include "FrameView.h"
#include "GraphicsContext.h"
#include "Image.h"
#include "HTMLNames.h"
#include "HTMLPlugInElement.h"
#include "KeyboardEvent.h"
#include "PlatformKeyboardEvent.h"
#include "MouseEvent.h"
#include "NotImplemented.h"
#include "Page.h"
#include "PlatformMouseEvent.h"
#include "PluginDebug.h"
#include "PluginPackage.h"
#include "RenderLayer.h"
#include "Settings.h"
#include "JSDOMBinding.h"
#include "ScriptController.h"
#include "npruntime_impl.h"
#include "runtime.h"
#include "runtime_root.h"

#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
//#include <FL/x.H>
extern bool fl_FLEventToDFBEvent(int e, DFBEvent& dfbevent);

using JSC::ExecState;
using JSC::Interpreter;
using JSC::JSObject;
using JSC::UString;

using std::min;

using namespace WTF;

namespace WebCore {

using namespace HTMLNames;

class Fl_PluginWindow : public Fl_Window {
    PluginView* m_pluginView;

public:    
    Fl_PluginWindow(PluginView* pluginView, int x, int y, int w, int h)
        : Fl_Window(x, y, w, h, "Fl_PluginWindow"), m_pluginView(pluginView)
    {}
    ~Fl_PluginWindow()
    {
        if (m_pluginView)
            m_pluginView->setPlatformWidget(NULL);
    }

    void draw() {
        /*
           IDirectFBSurface* surface = reinterpret_cast<IDirectFBSurface*>(getClientSurface());
           if (surface)
               surface->Clear(surface, 0, 0, 0, 0);
        */
    }

    int handle(int event) {
        if (!m_pluginView || !fl_dfb_event)
            return Fl_Window::handle(event);

        DFBEvent dfbevent;
        switch(event) {
            case FL_FOCUS:
            case FL_UNFOCUS:
                return 1;

        case FL_PUSH:
        case FL_RELEASE:
        case FL_MOVE:
        case FL_DRAG:
        case FL_SHORTCUT:
        case FL_KEYDOWN:
        case FL_KEYUP:
            if (fl_FLEventToDFBEvent(event, dfbevent)) {
            	if (dfbevent.window.key_symbol == DIKS_MENU)
            		return Fl_Window::handle(event);
            			
				if ((event == FL_KEYDOWN || event == FL_KEYUP || event == FL_SHORTCUT) && dfbevent.window.key_symbol == 0)
					dfbevent.window.key_symbol = (DFBInputDeviceKeySymbol)dfbevent.window.key_id;
                m_pluginView->sendDFBEventToGnash(&dfbevent);
                
            /*	if (dfbevent.window.key_symbol == DIKS_TITLE && event == FL_KEYDOWN)
            	{
            		DFBEvent dfbenter = dfbevent;
            		dfbenter.window.key_symbol = DIKS_RETURN;
            		dfbenter.window.key_id = DIKI_ENTER;
            		m_pluginView->sendDFBEventToGnash(&dfbenter);
            	}*/
	    	}
			
            return 1;
        default:
            return Fl_Window::handle(event);
        }
    }
};


void PluginView::updatePluginWidget()
{
    if (!parent() || !m_isWindowed)
        return;

    ASSERT(parent()->isFrameView());
    FrameView* frameView = static_cast<FrameView*>(parent());

    IntRect oldWindowRect = m_windowRect;
    IntRect oldClipRect = m_clipRect;

    m_windowRect = IntRect(frameView->contentsToWindow(frameRect().location()), frameRect().size());
    m_clipRect = windowClipRect();
    m_clipRect.move(-m_windowRect.x(), -m_windowRect.y());

    if (platformWidget() && (m_windowRect != oldWindowRect || m_clipRect != oldClipRect))
        setNPWindowRect(m_windowRect);
}

void PluginView::setFocus()
{
    if (Fl_PluginWindow* w = (Fl_PluginWindow*)platformWidget())
        Fl::focus(w);
}

void PluginView::show()
{
    if (Fl_PluginWindow* w = (Fl_PluginWindow*)platformWidget())
        w->show();
}

void PluginView::hide()
{
    if (Fl_PluginWindow* w = (Fl_PluginWindow*)platformWidget())
        w->hide();
}

/*void PluginView::paintMissingPluginIcon(GraphicsContext* context, const IntRect& rect)
{
    
    if (m_status !=  PluginStatusLoadedSuccessfully) {
        static PassRefPtr<WebCore::Image> image;
        if (!image)
            image = Image::loadPlatformResource("missing-plugin");
        
        IntRect imageRect(x(), y(), image->width(), image->height());
        int xOffset = (width() - imageRect.width()) / 2;
        int yOffset = (height() - imageRect.height()) / 2;
        imageRect.move(xOffset, yOffset);
        
        context->save();
        context->clip(windowClipRect());
        context->drawImage(image.get(), imageRect.location());
        context->restore();
    } else {
        static RefPtr<Image> nullPluginImage;
        if (!nullPluginImage)
            nullPluginImage = Image::loadPlatformResource("nullPlugin");

        IntRect imageRect(x(), y(), nullPluginImage->width(), nullPluginImage->height());

        int xOffset = (width() - imageRect.width()) / 2;
        int yOffset = (height() - imageRect.height()) / 2;

        imageRect.move(xOffset, yOffset);

        if (!rect.intersects(imageRect))
            return;

        context->save();
        context->clip(windowClipRect());
        context->drawImage(nullPluginImage.get(), imageRect.location());
        context->restore();
    }
}*/

void PluginView::paint(GraphicsContext* context, const IntRect& rect)
{
    //printf("%s {%d, %d, %d, %d}, {%d, %d, %d, %d}\n", __PRETTY_FUNCTION__, rect.x(), rect.y(), rect.width(), rect.height(), x(), y(), width(), height());
    FloatRect r(x(), y(), width(), height());
    context->clearRect(r);
	notImplemented();
    if (!m_isStarted || m_status !=  PluginStatusLoadedSuccessfully) {
        // Draw the "missing plugin" image
        paintMissingPluginIcon(context, rect);
        return;
    }
}

int PluginView::sendDFBEventToGnash(DFBEvent* event)
{
    int r = 0;

    printf("%s : pass dfb event into plugin %p\n", __FUNCTION__, m_plugin->pluginFuncs()->event);

    if (m_plugin->pluginFuncs()->event && event) {
	 	PluginView::setCurrentPluginView(this);
        setCallingPlugin(true);
		r = m_plugin->pluginFuncs()->event(m_instance, event);
		setCallingPlugin(false);
        PluginView::setCurrentPluginView(0);		
    }

    return r;
}

void PluginView::handleKeyboardEvent(KeyboardEvent* event)
{
    PlatformKeyboardEvent* platformEvent = event->platformKeyboardEvent();
	
    printf("%s : pass key event into plugin %p\n", __FUNCTION__, m_plugin->pluginFuncs()->event);
    //FIXME: Synthesize a SDLEvent to pass through
    if (m_plugin->pluginFuncs()->event && 
		platformEvent && 
		platformEvent->balEventKey()) {
		
	 	PluginView::setCurrentPluginView(this);
        	setCallingPlugin(true);
		if (!m_plugin->pluginFuncs()->event(m_instance, platformEvent->balEventKey() ))
        		event->setDefaultHandled();
		setCallingPlugin(false);
        	PluginView::setCurrentPluginView(0);		
    	}
}

void PluginView::handleMouseEvent(MouseEvent* event)
{
#if 0 //FELIX
    NPEvent npEvent;

    //FIXME: Synthesize a SDLEvent to pass through
    IntPoint p = static_cast<FrameView*>(parent())->contentsToWindow(IntPoint(event->pageX(), event->pageY()));

    if (m_plugin->pluginFuncs()->event && !m_plugin->pluginFuncs()->event(m_instance, &npEvent))
        event->setDefaultHandled();
#endif	
}

void PluginView::setParent(ScrollView* parent)
{
    Widget::setParent(parent);

    if (parent)
        init();
}

void PluginView::setNPWindowRect(const IntRect& rect)
{
    if (!m_isStarted || !parent())
        return;

    IntPoint p = static_cast<FrameView*>(parent())->contentsToWindow(rect.location());
    m_npWindow.x = p.x();
    m_npWindow.y = p.y();

    m_npWindow.width = rect.width();
    m_npWindow.height = rect.height();

    m_npWindow.clipRect.left = 0;
    m_npWindow.clipRect.top = 0;
    m_npWindow.clipRect.right = rect.width();
    m_npWindow.clipRect.bottom = rect.height();

    m_npWindow.window = platformWidget();

    printf("%s: {%d, %d, %d, %d}, setwindow = %p\n", __FUNCTION__, m_npWindow.x, m_npWindow.y, m_npWindow.width, m_npWindow.height, m_plugin->pluginFuncs()->setwindow);

//FIXME DLNA plugin will create fullscreen plugin.
#if 0
    if (Fl_PluginWindow* w = (Fl_PluginWindow*)platformWidget()) {
        w->resize(m_npWindow.x, m_npWindow.y, m_npWindow.width, m_npWindow.height);
    }
#endif    

    if (m_npWindow.x < 0 || m_npWindow.y < 0 ||
        m_npWindow.width <= 0 || m_npWindow.height <= 0)
        return;

    if (m_plugin->pluginFuncs()->setwindow) {
        PluginView::setCurrentPluginView(this);
        setCallingPlugin(true);
        m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
        setCallingPlugin(false);
        PluginView::setCurrentPluginView(0);

        if (!m_isWindowed)
            return;
    }
}

void PluginView::stop()
{
    if (!m_isStarted)
        return;

    HashSet<RefPtr<PluginStream> > streams = m_streams;
    HashSet<RefPtr<PluginStream> >::iterator end = streams.end();
    for (HashSet<RefPtr<PluginStream> >::iterator it = streams.begin(); it != end; ++it) {
        (*it)->stop();
        disconnectStream((*it).get());
    }

    ASSERT(m_streams.isEmpty());

    m_isStarted = false;

    // Clear the window
    m_npWindow.window = 0;
#ifdef XP_UNIX
    if (m_isWindowed && m_npWindow.ws_info)
           delete (NPSetWindowCallbackStruct *)m_npWindow.ws_info;
    m_npWindow.ws_info = 0;
#endif
    //FIXME setWindow will crash old youtube flash player plugin.
#if 0
    if (m_plugin->pluginFuncs()->setwindow && !m_plugin->quirks().contains(PluginQuirkDontSetNullWindowHandleOnDestroy)) {
        PluginView::setCurrentPluginView(this);
        setCallingPlugin(true);
        m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
        setCallingPlugin(false);
        PluginView::setCurrentPluginView(0);
    }
#endif    

    // Destroy the plugin
    {
        PluginView::setCurrentPluginView(this);
        setCallingPlugin(true);
        m_plugin->pluginFuncs()->destroy(m_instance, 0);
        setCallingPlugin(false);
        PluginView::setCurrentPluginView(0);
    }

    m_instance->pdata = 0;
}

static const char* MozillaUserAgent = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0";

const char* PluginView::userAgent()
{
    if (m_plugin->quirks().contains(PluginQuirkWantsMozillaUserAgent))
        return MozillaUserAgent;

    if (m_userAgent.isNull())
        m_userAgent = m_parentFrame->loader()->userAgent(m_url).utf8();

    return m_userAgent.data();
}

const char* PluginView::userAgentStatic()
{
    //FIXME - Lie and say we are Mozilla
    //    return MozillaUserAgent;
    return "";
}

void PluginView::invalidateRegion(NPRegion)
{ 
    notImplemented();
}

NPError PluginView::handlePostReadFile(Vector<char>& buffer, uint32 len, const char* buf)
{
    return NPERR_NO_ERROR;
}

NPError PluginView::getValue(NPNVariable variable, void* value)
{
    switch (variable) {
    case NPNVxDisplay:
        return NPERR_GENERIC_ERROR;

#if ENABLE(NETSCAPE_PLUGIN_API)
    case NPNVWindowNPObject: {
        if (m_isJavaScriptPaused)
            return NPERR_GENERIC_ERROR;

        NPObject* windowScriptObject = m_parentFrame->script()->windowScriptNPObject();
        // Return value is expected to be retained, as described here: <http://www.mozilla.org/projects/plugin/npruntime.html>
        if (windowScriptObject)
            _NPN_RetainObject(windowScriptObject);

        void** v = (void**)value;
        *v = windowScriptObject;

        return NPERR_NO_ERROR;
    }

    case NPNVPluginElementNPObject: {
        if (m_isJavaScriptPaused)
            return NPERR_GENERIC_ERROR;

        NPObject* pluginScriptObject = 0;

        if (m_element->hasTagName(appletTag) || m_element->hasTagName(embedTag) || m_element->hasTagName(objectTag))
        pluginScriptObject = static_cast<HTMLPlugInElement*>(m_element)->getNPObject();

        // Return value is expected to be retained, as described here: <http://www.mozilla.org/projects/plugin/npruntime.html>
        if (pluginScriptObject)
            _NPN_RetainObject(pluginScriptObject);

        void** v = (void**)value;
         *v = pluginScriptObject;

         return NPERR_NO_ERROR;
     }
#endif

     case NPNVnetscapeWindow:
     	return NPERR_NO_ERROR;

     default:
     	return getValueStatic(variable, value);
    }
}

NPError PluginView::getValueStatic(NPNVariable variable, void* value)
{
    switch (variable) {
    case NPNVToolkit:
        *((uint32 *)value) = 0;
        return NPERR_NO_ERROR;

    case NPNVSupportsXEmbedBool:
        *((uint32 *)value) = false;
        return NPERR_NO_ERROR;

    case NPNVjavascriptEnabledBool:
        *((uint32 *)value) = true;
        return NPERR_NO_ERROR;

    default:
        return NPERR_GENERIC_ERROR;
    }
}

void PluginView::invalidateRect(NPRect* rect)
{
    if (!rect) {
        invalidate();
        return;
    }

    IntRect r(rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top);
    Widget::invalidateRect(r);
}

void PluginView::forceRedraw()
{
}

PluginView::~PluginView()
{
    stop();

    deleteAllValues(m_requests);

    freeStringArray(m_paramNames, m_paramCount);
    freeStringArray(m_paramValues, m_paramCount);

    m_parentFrame->script()->cleanupScriptObjectsForPlugin(this);
   
    if (Fl_PluginWindow* w = (Fl_PluginWindow*)platformWidget()) {
        setPlatformWidget(NULL);
        if (w->parent())
            w->parent()->remove(w);
        delete w;
    }

    if (m_plugin && !(m_plugin->quirks().contains(PluginQuirkDontUnloadPlugin)))
        m_plugin->unload();
}

void PluginView::init()
{
    if (m_haveInitialized)
        return;
    m_haveInitialized = true;

    if (!m_plugin) {
        ASSERT(m_status == PluginStatusCanNotFindPlugin);
        return;
    }

    if (!m_plugin->load()) {
        m_plugin = 0;
        m_status = PluginStatusCanNotLoadPlugin;
        return;
    }

    if (!start()) {
        m_status = PluginStatusCanNotLoadPlugin;
        return;
    }

    if (m_plugin->pluginFuncs()->getvalue) {
        /*PluginView::setCurrentPluginView(this);
        setCallingPlugin(true);
        m_plugin->pluginFuncs()->getvalue(m_instance, NPPVpluginNeedsXEmbed, &m_needsXEmbed);
        setCallingPlugin(false);
        PluginView::setCurrentPluginView(0);*/
    }
    Fl_PluginWindow* w = new Fl_PluginWindow(this, 0, 0, 1, 1);
    Fl_Window* parent = (Fl_Window*)m_parentFrame->view()->hostWindow()->platformWindow();
    if (parent)
        parent->add(w);
    setPlatformWidget(w);
    show();

    if (m_isWindowed) {
        m_npWindow.type = NPWindowTypeWindow;
    }
    else {
        m_npWindow.type = NPWindowTypeDrawable;
        m_npWindow.window = 0;
    }

    if (!(m_plugin->quirks().contains(PluginQuirkDeferFirstSetWindowCall)))
    {
        updatePluginWidget();
        //LOG(Plugins, "PluginView::init frameRect = {%d, %d, %d, %d}\n", x(), y(), width(), height());
        //setNPWindowRect(frameRect());
    }

    m_status = PluginStatusLoadedSuccessfully;
}

void PluginView::invalidateRect(const IntRect& rect)
{
}

void PluginView::setParentVisible(bool visible)
{
}

} // namespace WebCore
