#include <config.h>
#include <PlatformString.h>
#include <CString.h>
#include <wtf/Threading.h>
#include <FileSystem.h>

#include "ATVWebDownload.h"
#include "WebFrame.h"

#if ENABLE(ACTVILA)
#include <acnavi.h>
#endif

static ATVWebDownload* s_instance = NULL;

ATVWebDownload* ATVWebDownload::getInstance()
{
    return s_instance;
}

ATVWebDownload::ATVWebDownload(WebView* webView)
    : mWebView(webView)
    , mHandle(NULL)
    , mCallback(NULL)
    , mContext(NULL)
{
    s_instance = this;
}

ATVWebDownload::~ATVWebDownload()
{
    stopDownload();
    s_instance = NULL;
}

void ATVWebDownload::startDownload(ResourceHandle* handle, const ResourceRequest* request, const ResourceResponse* response)
{
    printf("ATVWebDownload::startDownload: {%p, %p, %p}\n", handle, request, response);
    stopDownload();

    mHandle = handle;
    mHandle->setClient(this);
    didStart();
    didReceiveResponse(mHandle.get(), *response);
}

void ATVWebDownload::startDownload(const char* url, lpDownloadCallbackFunc callback, void* ctx)
{
    printf("ATVWebDownload::startDownload: {%s, %p, %p}\n", url, callback, ctx);
    stopDownload();

    ResourceRequest request(url);

    if (!mWebView || !mWebView->mainFrame())
        return;
    
    mHandle = ResourceHandle::create(request, this, core(mWebView->mainFrame()), false, false, false);
    mCallback = callback;
    mContext = ctx;

    didStart();
}

void ATVWebDownload::stopDownload()
{
    if (mHandle) {
        printf("ATVWebDownload::stopDownload\n");

        mHandle->setClient(NULL);
        mHandle->cancel();
        mHandle = NULL;

        mData.clear();

        mCallback = NULL;
        mContext = NULL;
    }
}

void ATVWebDownload::didStart()
{
    printf("ATVWebDownload::didStart\n");
}

void ATVWebDownload::didReceiveResponse(ResourceHandle*, const WebCore::ResourceResponse&)
{
    printf("ATVWebDownload::didReceiveResponse.\n");
}

void ATVWebDownload::didReceiveData(ResourceHandle* h, const char* data, int length, int lengthReceived)
{
    ASSERT(h == mHandle);

    printf("ATVWebDownload::didReceiveData: {%p, %d, %d}\n", data, length, lengthReceived);

    mData.append(data, length);
}

#if 0
static void writeToFile(char* data, int len)
{
    PlatformFileHandle tempFileHandle;
    CString path = openTemporaryFile("CPC", tempFileHandle);
    if (isHandleValid(tempFileHandle)) {
        printf("path = %s\n", path.data());
        writeToFile(tempFileHandle, data, len);
        closeFile(tempFileHandle);
    }
}
#endif

void ATVWebDownload::didFinishLoading(ResourceHandle* h)
{
    printf("ATVWebDownload::didFinishLoading: %d\n", mData.size());
    if (mHandle) {
        unsigned char* data = mData.data();
        size_t len = mData.size();

        printf("ATVWebDownload::didFinishLoading: %s, %p, %d\n", mHandle->request().url().string().utf8().data(), data, len);
        if (mCallback)
            mCallback(mContext, data, len);
#if ENABLE(ACTVILA)
        else //fullscreen playback
            AcNavi_StartVideo((char*)data, len, 0, 0, 0, 0, 0);        
#endif

        mHandle->setClient(NULL);
        mHandle = NULL;
        mCallback = NULL;
        mContext = NULL;
        mData.clear();
    }
}

void ATVWebDownload::didFail(ResourceHandle* h, const ResourceError& error)
{
    printf("ATVWebDownload::didFail: \n");
    if (mHandle) {
        mHandle->setClient(NULL);
        mHandle = NULL;        
    }
}

void ATVWebDownload::wasBlocked(ResourceHandle*)
{
    notImplemented();
}

void ATVWebDownload::cannotShowURL(ResourceHandle*)
{
    notImplemented();
}

