#include <config.h>
#include <wtf/Threading.h>

#include "WebKitATV.h"
#include "ATVWebDownload.h"
#include "WebFrame.h"
#include "WebView.h"

#include <FL/Fl.H>
#include <FL/Fl_Widget.H>

static void StartBrowser_Internal(void* data)
{
    const char* url = (const char*)data;
    ATVWebDownload* download = ATVWebDownload::getInstance();

    if (url && download && download->webView()) {
        WebView* view = download->webView();
        Fl_Widget* widget = view->viewWindow();

        if (widget && !widget->visible())
            widget->show();
        view->mainFrame()->loadURL(url);
    }
}

static void StopBrowser_Internal(void* data)
{
    bool* notify = (bool*)data;

    ATVWebDownload* download = ATVWebDownload::getInstance();

    if (download && download->webView()) {
        WebView* view = download->webView();
        Fl_Widget* widget = view->viewWindow();

        if (widget && widget->visible())
            widget->hide();
        view->mainFrame()->loadURL("about:blank");
    }

    if (notify)
        *notify = true;
}

////////////////// interface ////////////////////////////

int StartDownloadCPC(const char* url, lpDownloadCallbackFunc callback, void* ctx)
{
    printf("StartDownloadCPC: %s %p %p\n", url, callback, ctx);
    ATVWebDownload* download = ATVWebDownload::getInstance();
    if (download) {
        download->startDownload(url, callback, ctx);   
        return 0;
    }
    return -1;
}

int StopDownloadCPC()
{
    printf("StopDownloadCPC\n");
    ATVWebDownload* download = ATVWebDownload::getInstance();
    if (download) {
        download->stopDownload();   
        return 0;
    }
    return -1;
}

int StartBrowser(const char* URL)
{
    printf("StartBrowser: %s\n", URL);
    if (isMainThread()) {
        StartBrowser_Internal((void*)URL);
    } else {
        Fl::awake(StartBrowser_Internal, (void*)URL);
    }
    return 0;
}

int StopBrowser(void)
{
    printf("StopBrowser\n");

    if (isMainThread())
        StopBrowser_Internal(NULL);
    else {
        bool finished = false;
        Fl::awake(StartBrowser_Internal, &finished);
        while(!finished) {
            usleep(100);
            printf("StopBrowser: waiting browser exit\n");
        }
    }

    return 0;
}

