/*
 * Copyright (C) 2008 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DefaultPolicyDelegate_h
#define DefaultPolicyDelegate_h

#include <PlatformString.h>
#include <wtf/Vector.h>

class WebActionPropertyBag;
class WebFramePolicyListener;
class WebActionPropertyBag;
class WebMutableURLRequest;
class WebView;
class WebFrame;
class WebError;
class WebActionPropertyBag;
namespace WebCore {
    class String;
}


typedef enum WebNavigationType {
    WebNavigationTypeLinkClicked,
    WebNavigationTypeFormSubmitted,
    WebNavigationTypeBackForward,
    WebNavigationTypeReload,
    WebNavigationTypeFormResubmitted,
    WebNavigationTypeOther
} WebNavigationType;

#define WebActionNavigationTypeKey "WebActionNavigationTypeKey"
#define WebActionElementKey "WebActionElementKey"
#define WebActionButtonKey "WebActionButtonKey"
#define WebActionModifierFlagsKey "WebActionModifierFlagsKey"
#define WebActionOriginalURLKey "WebActionOriginalURLKey"


class DefaultPolicyDelegate {
public:

    /**
     * get shared instance
     * @param[out]: DefaultPolicyDelegate
     * @code
     * DefaultPolicyDelegate *d = DefaultPolicyDelegate::sharedInstance();
     * @endcode
     */
    static DefaultPolicyDelegate* sharedInstance();

    /**
     * create new instance of DefaultPolicyDelegate
     * @param[out]: DefaultPolicyDelegate
     * @code
     * DefaultPolicyDelegate *d = DefaultPolicyDelegate::createInstance();
     * @endcode
     */
    static DefaultPolicyDelegate* createInstance();
private:

    /**
     * DefaultPolicyDelegate default constructor
     */
    DefaultPolicyDelegate();
public:

    /**
     * DefaultPolicyDelegate destructor
     */
    virtual ~DefaultPolicyDelegate();

    /**
     * decide policy for navigation action
     * @param[in]: WebView
     * @param[in]: WebActionPropertyBag
     * @param[in]: WebMutableURLRequest
     * @param[in]: WebFrame
     * @param[in]: WebFramePolicyListener
     * @code
     * d->decidePolicyForNavigationAction(webView, actionInformation, request, frame, listener);
     * @endcode
     */
    virtual void decidePolicyForNavigationAction( 
        /* [in] */ WebView *webView,
        /* [in] */ WebActionPropertyBag *actionInformation,
        /* [in] */ WebMutableURLRequest *request,
        /* [in] */ WebFrame *frame,
        /* [in] */ WebFramePolicyListener *listener);
    
    /**
     * decide policy for new window action
     * @param[in]: WebView
     * @param[in]: WebActionPropertyBag
     * @param[in]: WebMutableURLRequest
     * @param[in]: frame name
     * @param[in]: WebFramePolicyListener
     * @code
     * d->decidePolicyForNewWindowAction(webView, actionInformation, request, frameName, listener);
     * @endcode
     */
    virtual void decidePolicyForNewWindowAction( 
        /* [in] */ WebView *webView,
        /* [in] */ WebActionPropertyBag *actionInformation,
        /* [in] */ WebMutableURLRequest *request,
        /* [in] */ WebCore::String frameName,
        /* [in] */ WebFramePolicyListener *listener);
    
    /**
     * decide policy for MIMEType
     * @param[in]: WebView
     * @param[in]: type
     * @param[in]: WebMutableURLRequest
     * @param[in]: WebFrame
     * @param[in]: WebFramePolicyListener
     * @code
     * d->decidePolicyForMIMEType(webView, type, request, frame, listener);
     * @endcode
     */
    virtual void decidePolicyForMIMEType( 
        /* [in] */ WebView *webView,
        /* [in] */ const char* type,
        /* [in] */ WebMutableURLRequest *request,
        /* [in] */ WebFrame *frame,
        /* [in] */ WebFramePolicyListener *listener);
    
    /**
     * unable to implement policy with error
     * @param[in]: webView
     * @param[in]: error
     * @param[in]: frame
     * @code
     * d->unableToImplementPolicyWithError(webView, error, frame);
     * @endcode
     */
    virtual void unableToImplementPolicyWithError( 
        /* [in] */ WebView *webView,
        /* [in] */ WebError *error,
        /* [in] */ WebFrame *frame);

private:
    Vector<WebCore::String> mLockedSites;
    Vector<WebCore::String> mLimitSites;
};

#endif // DefaultPolicyDelegate_h
