
#include "WebPreferences.h"
#include "WebCookieJar.h"
#include <map>
using namespace std;

class WebCookieJarPrivate {
public:
    map<string, string> mCookiesJar;
};

WebCookieJar* WebCookieJar::webCookieJar()
{
    static WebCookieJar* cookieJar;
    if (!cookieJar)
        cookieJar = new WebCookieJar();
    return cookieJar;
}

WebCookieJar::WebCookieJar()
    : d(new WebCookieJarPrivate())
{
}

WebCookieJar::~WebCookieJar()
{
    delete d;
}

void WebCookieJar::setCookies(const string& url, const string& cookies)
{
    d->mCookiesJar[url] = cookies;
}

string WebCookieJar::cookies(const string& url)
{
    map<string, string>::iterator it = d->mCookiesJar.find(url);
    if (it != d->mCookiesJar.end())
        return it->second;
    return string("");
}

void WebCookieJar::setEnabled(bool enabled)
{
    WebPreferences* pref = WebPreferences::sharedStandardPreferences();
    if (pref)
        pref->setCookiesEnabled(enabled);
}

bool WebCookieJar::isEnabled() const
{
    WebPreferences* pref = WebPreferences::sharedStandardPreferences();
    if (pref)
        return pref->cookiesEnabled();
    return true;
}

void WebCookieJar::copyKeysToVector(vector<string>& keys)
{
    map<string, string>::iterator it;
    map<string, string>::iterator end;

    keys.reserve(d->mCookiesJar.size());
    it = d->mCookiesJar.begin();
    end = d->mCookiesJar.end();
    for(unsigned i = 0; it != end; ++it, i++)
        keys.push_back(it->first);
}

void WebCookieJar::clear()
{
	d->mCookiesJar.clear();
}

