/*
 * Copyright (C) 2007 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef BINDINGS_BAL_OBJECT_H_
#define BINDINGS_BAL_OBJECT_H_

#include <string>
#include <vector>
#include "WebKitDefines.h"

class WebObject;
class WebValuePrivate;

using namespace std;

class WEBKIT_OWB_API WebValue {
    public:
        WebValue();
        WebValue(WebValuePrivate *priv);
        ~WebValue();

        bool isUndefined() const;
        bool isNull() const;
        bool isUndefinedOrNull() const;
        bool isBoolean() const;
        bool isNumber() const;
        bool isString() const;
        bool isGetterSetter() const;
        bool isObject() const;

        bool toBoolean() const;
        double toNumber() const;
        const char* toString() const;
        WebObject *toObject() const;
        ///return value converted from string color (named or valued) to binary color. returns 0x0 if problem when decoding color
        unsigned int toRGBA32() const;

        void balUndefined();
        void balNull();
        void balNaN();
        void balBoolean(bool b);
        void balNumber(double d);
        void balString(const char* s);
        void balObject(WebObject *obj);

        WebValuePrivate *d;
        WebObject *m_obj;
};

class BalClass;
class BalMethod;

class WEBKIT_OWB_API WebObject
{
public:
    WebObject();
    virtual ~WebObject();
    void invalidate();
    bool hasMethod(const char *name);
    virtual const char* getName() { return ""; };
    virtual WebValue *invoke(const char *name, vector<WebValue *> args);
    bool hasProperty(const char *name);
    virtual WebValue *getProperty(const char *name);
    virtual void setProperty( const char *name, WebValue *value);
    void addMethod(const char *);
    void removeMethod(const char *);
    void addProperty(const char *);
    void removeProperty(const char *);
    size_t methodCount() {return m_balMethodList.size();}
    size_t propertyCount() {return m_balPropertyList.size();}
private:
    vector<string> m_balMethodList;
    vector<string> m_balPropertyList;
};

#endif
