/*
 * Copyright (C) 2007 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef BINDINGS_BALVALUEPRIVATE_H_
#define BINDINGS_BALVALUEPRIVATE_H_

#include "JSValue.h"
#include <interpreter/CallFrame.h>
#include "WebObject.h"
#include "PlatformString.h"
#include "CString.h"
#include "JSGlobalObject.h"
#include "runtime_object.h"
#include "bal_instance.h"

using namespace JSC;
class WebValuePrivate {
    public:
        WebValuePrivate()
        {
            PassRefPtr<JSGlobalData> sharedGlobalData = JSGlobalData::create();
            JSGlobalObject* obj = new (sharedGlobalData.get()) JSGlobalObject();
            m_exec = obj->globalExec();
        }
        WebValuePrivate(ExecState* exec, JSValue value)
        {
            m_val = value;
            m_exec = exec;
        }

        ~WebValuePrivate()
        {
            m_exec = 0;
        }

        bool isUndefined() const;
        bool isNull() const;
        bool isUndefinedOrNull() const;
        bool isBoolean() const;
        bool isNumber() const;
        bool isString() const;
        bool isGetterSetter() const;
        bool isObject() const;

        bool toBoolean() const;
        double toNumber() const;
        const char* toString() const;
        WebObject *toObject() const;

        void balUndefined();
        void balNull();
        void balNaN();
        void balBoolean(bool b);
        void balNumber(double d);
        void balString(const char* s);
        JSObject* balObject(WebObject *obj, ExecState *exec);

        JSValue getValue() { return m_val; }

    private:
        JSValue m_val;
        ExecState *m_exec;
};


inline bool WebValuePrivate::isUndefined() const
{
    return m_val.isUndefined();
}

inline bool WebValuePrivate::isNull() const
{
    return m_val.isNull();
}

inline bool WebValuePrivate::isUndefinedOrNull() const
{
    return m_val.isUndefinedOrNull();
}

inline bool WebValuePrivate::isBoolean() const
{
    return m_val.isBoolean();
}

inline bool WebValuePrivate::isNumber() const
{
    return m_val.isNumber();
}

inline bool WebValuePrivate::isString() const
{
    return m_val.isString();
}

inline bool WebValuePrivate::isGetterSetter() const
{
    return m_val.isGetterSetter();
}

inline bool WebValuePrivate::isObject() const
{
    return m_val.isObject();
}

inline bool WebValuePrivate::toBoolean() const
{
    return m_val.toBoolean(m_exec);
}

inline double WebValuePrivate::toNumber() const
{
    return m_val.toNumber(m_exec);
}

inline const char* WebValuePrivate::toString() const
{
    WebCore::String st(m_val.toString(m_exec).data());
    return st.utf8().data();
}

inline WebObject *WebValuePrivate::toObject() const
{
    JSObject* object = m_val.toObject(m_exec);
    if (object->classInfo() == &RuntimeObjectImp::s_info) {
        RuntimeObjectImp* imp = static_cast<RuntimeObjectImp *>(object);
        Bindings::BalInstance* instance = static_cast<Bindings::BalInstance*>(imp->getInternalInstance());
        if (instance) {
            WebObject* obj = instance->getObject();
            return obj;
        }
    }
    return NULL;
}

inline void WebValuePrivate::balUndefined()
{
    m_val = jsUndefined();
}

inline void WebValuePrivate::balNull()
{
    m_val = jsNull();
}

inline void WebValuePrivate::balNaN()
{
    m_val = jsNaN(m_exec);
}

inline void WebValuePrivate::balBoolean(bool b)
{
    m_val = jsBoolean(b);
}

inline void WebValuePrivate::balNumber(double d)
{
    m_val = jsNumber(m_exec, d);
}

inline void WebValuePrivate::balString(const char* s)
{
    m_val = jsString(m_exec, s);
}

inline JSObject* WebValuePrivate::balObject(WebObject *obj, ExecState *exec)
{
    return JSC::Bindings::BalInstance::getBalInstance(obj, Bindings::findRootObject(exec->dynamicGlobalObject()))->createRuntimeObject(exec);
    //return Bindings::Instance::createRuntimeObject(exec, Bindings::BalInstance::create(obj, Bindings::findRootObject(exec->dynamicGlobalObject())));
}

#endif
