#include <DumpRenderTree.h>
#include <WebKit.h>
#include <FL/x.H>
#include "Fl_WebViewWindow.h"


// Choose some default values.
const unsigned int maxViewWidth = 960;
const unsigned int maxViewHeight = 540;
unsigned int waitToDumpWatchdog = 0;

//static WebView* webView;
static Fl_WebViewWindow* viewWindow;

void init(int argc, char *argv[])
{
    fl_open_display();
}

BalRectangle clientRect(bool isSVGW3CTest)
{
    DFBRectangle r;
    r.x = r.y = 0;
    r.w = isSVGW3CTest ? 480 : maxViewWidth;
    r.h = isSVGW3CTest ? 360 : maxViewHeight;
    return r; 
}

BalWidget* createWindow(WebView *webView, BalRectangle rect)
{
    int w = rect.w;
    int h = rect.h;

    viewWindow = new Fl_WebViewWindow(w, h);    
    viewWindow->setWebView(webView);
    webView->setViewWindow(viewWindow);
    return viewWindow;
}

void waitEvent()
{
    while (!done) {
        Fl::wait(0.1);
    }
}

void startEventLoop(BalWidget *view)
{
    done = false;

    waitEvent();
}

void stopEventLoop()
{
    done = true;
}

static void waitToDumpWatchdogFired(void* param)
{
    const char* message = "FAIL: Timed out waiting for notifyDone to be called\n";
    fprintf(stderr, "%s", message);
    fprintf(stdout, "%s", message);
    waitToDumpWatchdog = 0;
    dump();
}


void addTimetoDump(int timeoutSeconds)
{
    Fl::add_timeout(timeoutSeconds * 1000, waitToDumpWatchdogFired);
    waitToDumpWatchdog = 1;
}

void removeTimer()
{
    Fl::remove_timeout(waitToDumpWatchdogFired);
    waitToDumpWatchdog = 0;
}
