
#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include "Fl_DigitalClock.h"

#include <stdio.h>

static void tick(void* v) {
    reinterpret_cast<Fl_Digital_Clock*>(v)->setTime(time(0));
    Fl::add_timeout(1.0, tick, v);
}

Fl_Digital_Clock::Fl_Digital_Clock(int x, int y, int w, int h, const char* l, int level)
    : Fl_Box(x, y, w, h, l)
    , mLevel(level)
    , mCurrentTime(0)
{}

Fl_Digital_Clock::~Fl_Digital_Clock()
{
    Fl::remove_timeout(tick, this);
}

void Fl_Digital_Clock::setTime(time_t time)
{
    mCurrentTime = time;
    damage(FL_DAMAGE_ALL);
}

int Fl_Digital_Clock::handle(int event)
{
    switch(event) {
        case FL_SHOW:
            tick(this);
            break;
        case FL_HIDE:
            Fl::remove_timeout(tick, this);
            break;
    }
    return Fl_Widget::handle(event);
}

void Fl_Digital_Clock::draw()
{
    char* time;
    time = ctime(&mCurrentTime);
#if 1
    label(time);
    Fl_Box::draw();
#else
    Fl_Box::draw_box();
    // Draw label w/dropshadow effect
    fl_font(labelfont(), labelsize());
    int X = x() + Fl::box_dx(box());
    int Y = y() + Fl::box_dy(box());
    for (int i = mLevel; i > 0; i--) {// loop through shades of color
        fl_color(fl_color_average(color(), // bg color (widget's bg)
                    labelcolor(),          // fg color (label)
                    (i / (float)mLevel)));       // weight between bg and fg color
        fl_draw(time, X+i, Y+i, w(), h(), align());
    }
#endif    
}


