#include <stdio.h>

#include "Fl_InputKB.h"

Fl_InputKB::Fl_InputKB(int x, int y,int w, int h,const char* label)
	:Fl_Input(x,y,w,h,label)
{
	mXLimit = -1;
	mX = x;
	mMaxWidth = 10000;
}

Fl_InputKB::~Fl_InputKB()
{
}

int Fl_InputKB::handle(int event)
{
	switch (event) {
		case FL_KEYBOARD:
		case FL_KEYUP:		
			return 0;
		default:
			return Fl_Input::handle(event);	 
	}	
}

void Fl_InputKB::cutAsUTF8(int n)
{
	int pos = getPositionAsUTF8(n);
	cut(pos);
}

int Fl_InputKB::moveCursorAsUTF8(int n)
{
	int pos = getPositionAsUTF8(n);
	position(position()+pos);
}

void Fl_InputKB::move(int pos_x, int pos_y)
{
	int yy = (pos_y == -1)? y(): pos_y;
	mX = pos_x;
	resize(pos_x, yy, w(), h());
	//printf("move to x = %d, y = %d.\n", pos_x, yy);
}

void Fl_InputKB::setXLimit(int x)
{
	mXLimit = x;
}

void Fl_InputKB::insertAndAdjustSize(const char* t, int l)
{
	mark(position());	// mark to the end of the input.
	
	int width = w() + 20*l;
	size(width, h());
	insert(t, l);
	updateCursorInfo();
	width = get_cursor_x() - x() + 1;
	if(width >= mMaxWidth)
		width = mMaxWidth;

	// the min width is 20.
	if(width < 20) width = 20;

	// if the x limit is set.
	if(mXLimit != -1){
		int xx = x() + width - mXLimit;
		if(xx > 0){
			resize(x()-xx, y(), width, h());
			return;
		}
	}
	
	size(width, h());
	//printf("inset and adjust size (w = %d) ( x = %d).\n", width, x());
}

void Fl_InputKB::highlightLastCharacter()
{
	Fl::focus(this);
	int pos = getPositionAsUTF8(-1);
	mark(position()+pos);
}

void Fl_InputKB::cutAndAdjustSize(int n)
{
	cutAsUTF8(n);
	updateCursorInfo();
	int width = get_cursor_x() - x() + 1;
	if(width >= mMaxWidth)
		width = mMaxWidth;
		
	if(mX == x()){
		resize(x(), y(), width, h());
	}
	else {
		// if the x limit is set.
		if(mXLimit != -1){
			int xx = mX + width - mXLimit;
			if(xx > 0){
				resize(mX-xx, y(), width, h());
				return;
			}
		}
		resize(mX, y(), width, h());
	}
}

void Fl_InputKB::updateCursorInfo()
{
	calcTextLayout(x(), y(), w(), h());	
}

void Fl_InputKB::setMaxWidth(int x)
{
	mMaxWidth = x > 0 ? x : 0;
}

/*
int Fl_InputKB::getPositionWidth()
{
	float w = 0.0;
	const char * data_buf = value();
	int data_len = position();
	int text_size = textsize();
	const char * word_start = data_buf;
	int word_len = 1;
	for(int i = 0; i < data_len; ++i){
		bool be_first = true;
		if((data_buf[i] & 0x80) == 0x80 && (data_buf[i] & 0xc0) != 0xc0){	// not the first byte of utf8.
			++word_len;
			be_first = false;
		}

		if(i == 0 && i != data_len - 1){
			// ignore first letter.
		}
		else if(be_first || i == data_len -1){
			//int len = getStrWidth(word_start, word_len) ;
			int len = 0;
			printf("get the width of first character of (%s, len = %d) = %d.\n", word_start, word_len, len);
			if(len > 16) len = 16;
			w += len * 20 / 14;
			word_start = &data_buf[i];
			word_len = 1;
		}
	}
	
	printf("inputKB : get position width = %f.\n", w);
	return (int)w;
}
*/

// if n >= 0, get the position from the beginning.
// if n < 0, get the position from the end.
int Fl_InputKB::getPositionAsUTF8(int n)
{
	const char * data = value();
	int old_pos = position();
	int pos = old_pos;
	int remain = n;
	if(n >= 0){
		int len = size();
		if(old_pos == len) return 0;
		for(; pos < len; ++pos){
			if((data[pos] & 0x80) == 0x80 && (data[pos] & 0xc0) != 0xc0){	// not the first byte of utf8.
			}
			else {
				--remain;
				if(remain < 0) break;
			}			
		}
	}
	else {
		if(old_pos == 0) return 0;
		for(; pos >= 0; --pos){
			if((data[pos] & 0x80) == 0x80 && (data[pos] & 0xc0) != 0xc0){
			}
			else {
				++remain;
				if(remain > 0) break;
			}
		}
	}
	//printf("get move len = %d.\n", pos - old_pos);
	return pos - old_pos;
}

