
#include <string>
using std::string;

#include <stdlib.h>
#include <string.h>

#include <FL/Fl_Menu_.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Menu_Button.H>

#include "Fl_MainMenu.h"
#include "MainApplication.h"
#include "Fl_SoftKeyboard.h"
#include "Fl_WebViewWindow.h"

#include "History.h"
#include "Bookmark.h"
#include "WebCookieJar.h"
#include "WebPreferencesInterface.h"


static Fl_Menu_Item sHistoryMenu[History::MAX_HISTORY_ITEMS + 1];

static const int MAX_BOOKMARK_ITEMS = 20;

static Fl_Menu_Item sBookmarkMenu[MAX_BOOKMARK_ITEMS];

static Fl_Menu_Item sDeleteMenu[] = {
    {"Delete All", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {"Delete Cookies", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {"Delete History", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {"Delete Favourite", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {0,0,0,0,0,0,0,0,0}
};

static Fl_Menu_Item sCookieMenu[] = {
    {" Enable   ", 0,  0, (void*)1, FL_MENU_RADIO | FL_MENU_VALUE, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {" Disable  ", 0,  0, (void*)2, FL_MENU_RADIO, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {0,0,0,0,0,0,0,0,0},
};

static Fl_Menu_Item sJavaScriptMenu[] = {
    {" Enable   ", 0,  0, (void*)1, FL_MENU_RADIO | FL_MENU_VALUE, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {" Disable  ", 0,  0, (void*)2, FL_MENU_RADIO, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
    {0,0,0,0,0,0,0,0,0},
};

static Fl_Menu_Item sMainMenu[] = {
 {"   Cookies         ", 0, 0, (void*)sCookieMenu, FL_SUBMENU_POINTER, FL_NORMAL_LABEL, 0, 18, FL_WHITE}, 
 {"   JavaScript      ", 0, 0, (void*)sJavaScriptMenu, FL_SUBMENU_POINTER, FL_NORMAL_LABEL, 0, 18, FL_WHITE}, 
 {"   Delete          ", 0, 0, (void*)sDeleteMenu, FL_SUBMENU_POINTER, FL_NORMAL_LABEL, 0, 18, FL_WHITE}, 
 {"   History         ", 0,  0, (void*)sHistoryMenu, FL_SUBMENU_POINTER, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
 {"   Favourite       ", 0,  0, (void*)sBookmarkMenu, FL_SUBMENU_POINTER, FL_NORMAL_LABEL, 0, 18, FL_WHITE},
 {0,0,0,0,0,0,0,0,0},
};

void Delete_cb(Fl_Widget* w, void*) {
  Fl_Menu_Item* item = (Fl_Menu_Item*)w;
  
  MainApplication* app = MainApplication::application();
  if(item)
  {
  	if(strcmp(item->label(),"Delete All")==0)
  	{
  		WebCookieJar* cookies = app->cookieJar();
  		cookies->clear();	
  		
  		History* history = app->history();
  		history->removeAllItems();
  		
  		Bookmark* bookmark = app->bookmark();
  		bookmark->removeAllItems();	
  	}
  	if(strcmp(item->label(),"Delete Cookies")==0)
  	{
  		WebCookieJar* cookies = app->cookieJar();
  		cookies->clear();
  	}	
  	if(strcmp(item->label(),"Delete History")==0)
  	{	
  		History* history = app->history();
  		history->removeAllItems();
  	}
  	if(strcmp(item->label(),"Delete Favourite")==0)
  	{	
  		Bookmark* bookmark = app->bookmark();
  		bookmark->removeAllItems();	
  	}
  }	
}
bool isHistoryItem(Fl_Menu_Item* item)
{
    return (item >= &sHistoryMenu[0] && item <= &sHistoryMenu[History::MAX_HISTORY_ITEMS]);
}

bool isBookmarkItem(Fl_Menu_Item* item)
{
    return (item >= &sBookmarkMenu[0] && item <= &sBookmarkMenu[MAX_BOOKMARK_ITEMS]);
}

static void handleHistoryMenu(Fl_Menu_Item* item)
{
    MainApplication* app = MainApplication::application();
    WebView* webView = app->webViewWindow()->webView();
    History* history = app->history();
    
    if (!item || !webView || !history)
        return;

    int index = item - &sHistoryMenu[0];
    string title, url;
    history->itemAt(index, title, url);
    if (!url.empty())
        webView->mainFrame()->loadURL(url.c_str());
}

static void handleBookmarkMenu(Fl_Menu_Item* item)
{
    MainApplication* app = MainApplication::application();
    WebView* webView = app->webViewWindow()->webView();
    
    Bookmark* bookmark = app->bookmark();
    
    if (!item || !webView || !bookmark)
        return;

    int index = item - &sBookmarkMenu[0];
    string title, url;
    bookmark->itemAt(index, title, url);
    if (!url.empty())
        webView->mainFrame()->loadURL(url.c_str());
}

void HandleSubItem_cb(Fl_Widget* w, void*) {
	Fl_Menu_Item* item = (Fl_Menu_Item*)w;
	
	MainApplication* app = MainApplication::application();
	WebView* webView = app->webViewWindow()->webView();
	
  	if (!item || !webView)
        return;

    if (isHistoryItem(item)) {
        handleHistoryMenu(item);
        return;
    }
    if (isBookmarkItem(item)) {
        handleBookmarkMenu(item);
        return;
    }
    if (item == &sCookieMenu[0] || item == &sCookieMenu[1]) {
        WebCookieJar* cookieJar = app->cookieJar();
        if (cookieJar)
            cookieJar->setEnabled(item == &sCookieMenu[0]);
        webView->setCookieEnabled(item == &sCookieMenu[0]);
        return;
    }
    if (item == &sJavaScriptMenu[0] || item == &sJavaScriptMenu[1]) {
        WebPreferencesInterface* pref = app->preferences();
        if (pref)
            pref->setJavaScriptEnabled(item == &sJavaScriptMenu[0]);
        return;
    }
}

static int ButtonX[ButtonArraySize] = {0,110,210,310,410,510,610,710,810};
static char* ButtonImageActive[ButtonArraySize] = {"button-hover-01.png","button-hover-02.png","button-hover-03.png","button-hover-04.png","button-hover-05.png",
	                                               "button-hover-06.png","button-hover-07.png","button-hover-08.png","button-hover-09.png"};
static char* ButtonImageInactive[ButtonArraySize] = {"button01.png","button02.png","button03.png","button04.png","button05.png","button06.png",
	                                               "button07.png","button08.png","button09.png"};

static void buttonCallback(Fl_Widget* w, void*data)
{
    Fl_MainMenu* menu = static_cast<Fl_MainMenu*>(data);
    if (menu)
        menu->handleMenu();
}

static void mainMenuCallback(Fl_Widget* w, void*)
{
    Fl_MainMenu* menu = static_cast<Fl_MainMenu*>(w);
    if (menu)
        menu->handleMenu();
}

Fl_MainMenu::Fl_MainMenu(int x, int y, int w, int h, const char* label)
    : Fl_Group(x, y, w, h, label)
    , mPopuping(false)
{
    this->callback(mainMenuCallback, this);
    
    prepareButtonMenu(y); 
    
    sCookieMenu[0].callback(HandleSubItem_cb);
    sCookieMenu[1].callback(HandleSubItem_cb);
    
    sJavaScriptMenu[0].callback(HandleSubItem_cb);
    sJavaScriptMenu[1].callback(HandleSubItem_cb);
    
    sDeleteMenu[0].callback(Delete_cb);
    sDeleteMenu[1].callback(Delete_cb);
    sDeleteMenu[2].callback(Delete_cb);
    sDeleteMenu[3].callback(Delete_cb);
   
    m_advance = new Fl_Menu_Bar(810,480,0,0);
    m_advance->color(FL_BLACK);                          //set popupmenu's bg
    m_advance->box(FL_FLAT_BOX);                         //set popupmenu's box type
    m_advance->selection_color(fl_rgb_color(28,182,244));//set popupmenu's selection color
}

Fl_MainMenu::~Fl_MainMenu()
{
	if(m_advance != NULL)
		delete m_advance;
	
	for(int i = 0; i < ButtonArraySize; i++)
	{
		if (m_GButton[i] != NULL)
			delete m_GButton[i];
	}
}

void Fl_MainMenu::prepareButtonMenu(int StartY)
{	
	for(int i = 0; i < ButtonArraySize; i++)
	{
		m_GButton[i] = new Fl_GraphicButton(ButtonX[i],StartY,0,0,0,ButtonImageInactive[i],ButtonImageActive[i]);
        m_GButton[i]->color(fl_rgb_color(0, 0, 0));
        m_GButton[i]->callback(buttonCallback, this);
	}
}


int Fl_MainMenu::handle(int event) {
    switch(event) {
        case FL_FOCUS:
        case FL_UNFOCUS:
            return 1;           
           
        case FL_KEYDOWN:
            {
            	int key = Fl::event_key();
				
				if((key == FL_Left && Fl::focus() == m_GButton[0]) || 
				   (key == FL_Right && Fl::focus() == m_GButton[8]))
				   	return 1;
				if(key == FL_Up || key == FL_Down)
					return 1;   	   	
            }

        /*mouse event process    */   
        case FL_PUSH:
        	 {      	 	
        	 	for(int i=0;i<ButtonArraySize;i++)
        	 	{	
        	 		if(Fl::event_inside(m_GButton[i]))	
             		{
             			m_GButton[i]->take_focus();
             			this->do_callback((Fl_Widget*)this);
             			return 1;
             		}
            	}
            }
              
        default:
            return Fl_Group::handle(event);
    }
}

int Fl_MainMenu::popupMenu()
{
    if (mPopuping) {
        printf("%s : is popuping\n", __FUNCTION__);
        return 0;
    }
    
    if (m_GButton[0]==0) {
        printf("%s : null menu button \n", __FUNCTION__);
        return 0;
    }
    
    mPopuping = true;

    //const Fl_Menu_Item* v = menu()->first();
    //v = menu()->pulldown(x(), y(), w(), h(), v, this, 0, 1);
    //picked(v);
    MainApplication* app = MainApplication::application();

#if 0    	
    if(Fl::focus()==app->webViewWindow())
    {
    	m_GButton[0]->take_focus();
		updateMainMenu();
	}else{
		app->webViewWindow()->take_focus();
	}
#else
    int index=0;
    for(index = 0; index < ButtonArraySize; index++)
    {
    	if(Fl::focus()==m_GButton[index])
    	{
    		app->webViewWindow()->take_focus();
    		break;
		}	
    }
    if(index == 9){
    	m_GButton[0]->take_focus();
    }
#endif		
	
    mPopuping = false;
    return 1;
}

void Fl_MainMenu::updateMainMenu()
{
    updateSettingsMenu();
    updateHistoryMenu();
    updateBookmarkMenu();
}

void Fl_MainMenu::updateHistoryMenu()
{

    for(unsigned int i = 0; i < sizeof(sHistoryMenu) / sizeof(sHistoryMenu[0]); i++) {
        char* title = const_cast<char*>(sHistoryMenu[i].label());
        if (!title)
            break;
        free(title);
        sHistoryMenu[i].label(0);
    }
    History* history = MainApplication::application()->history();
    if (!history) {
        printf("null history \n");
        return;
    }
    
    for(unsigned int i = 0; i < history->countItems(); i++){
        string title;
        string url;
        history->itemAt(i, title, url);
        sHistoryMenu[i].label(strdup(title.c_str()));
        sHistoryMenu[i].labelsize(18);
        sHistoryMenu[i].labelcolor(FL_WHITE);
        sHistoryMenu[i].callback(HandleSubItem_cb);
    }
}

void Fl_MainMenu::updateBookmarkMenu()
{
    for(unsigned int i = 0; i < sizeof(sBookmarkMenu) / sizeof(sBookmarkMenu[0]); i++) {
        char* title = const_cast<char*>(sBookmarkMenu[i].label());
        if (!title)
            break;
        free(title);
        sBookmarkMenu[i].label(0);
    }
    Bookmark* bookmark = MainApplication::application()->bookmark();
    if (!bookmark) {
        printf("null bookmark \n");
        return;
    }
    
    for(unsigned int i = 0; i < bookmark->countItems() && i < MAX_BOOKMARK_ITEMS; i++){
        string title;
        string url;
        bookmark->itemAt(i, title, url);
        sBookmarkMenu[i].label(strdup(title.c_str()));
        sBookmarkMenu[i].labelsize(18);
        sBookmarkMenu[i].labelcolor(FL_WHITE);
        sBookmarkMenu[i].callback(HandleSubItem_cb);
    }
}

void Fl_MainMenu::updateSettingsMenu()
{
    WebPreferencesInterface* pref = MainApplication::application()->preferences();
    if (pref) {
        if (pref->cookiesEnabled()) {
            sCookieMenu[0].set();
            sCookieMenu[1].clear();
        } else {
            sCookieMenu[1].set();
            sCookieMenu[0].clear();
        }
        if (pref->isJavaScriptEnabled()) {
            sJavaScriptMenu[0].set();
            sJavaScriptMenu[1].clear();
        } else {
            sJavaScriptMenu[1].set();
            sJavaScriptMenu[0].clear();
        }
    }
}

void Fl_MainMenu::handleMenu()
{
    MainApplication* app = MainApplication::application();
    WebView* webView = app->webViewWindow()->webView();
    
    Fl_GraphicButton* gb = (Fl_GraphicButton*)Fl::focus() ;
      
    if(!gb || !webView)
    	return;
      
	if (gb == m_GButton[0]) 
   	{
        Fl_SoftKeyboard* softKeyboard = app->softKeyboard();
        if (softKeyboard) {
            const char* url = softKeyboard->showSoftKeyboard("http://",false,207,88);
            if (url && url[0] != 0) {
                std::string uriString(url);
                if (strncmp(url, "http://", 7))
                    uriString = "http://" + uriString;
                webView->mainFrame()->loadURL(uriString.c_str());
            }
        }
    } else if (gb == m_GButton[1]) {
        webView->goForward();
    } else if (gb == m_GButton[2]) {
        webView->goBack();
    } else if (gb == m_GButton[3]) {
        webView->refresh();
    } else if (gb == m_GButton[4]) {
        webView->zoomPageIn();
    } else if (gb == m_GButton[5]) {
        webView->zoomPageOut();
    } else if (gb == m_GButton[6]) {
        webView->mainFrame()->loadURL("http://www.google.com");
    } else if (gb == m_GButton[7]) {
        Bookmark* bookmark = app->bookmark();
        if (bookmark) {
            string title = webView->mainFrameTitle();
            string url = webView->mainFrameURL();          
            bookmark->addItem(title, url);
            app->messageInfo("%s has been added to Favourite?", webView->mainFrameTitle());
        }
    } else if (gb == m_GButton[8]) {
    	updateMainMenu();
    	const Fl_Menu_Item* MItem;
		if(sMainMenu)
		{
			MItem = sMainMenu->popup(810, 480,NULL,MItem,(Fl_Menu_ *)m_advance);
			
			if(MItem ==NULL)
				m_GButton[8]->take_focus();
			
			if(MItem&& MItem->submenu() == false)
				MItem->do_callback((Fl_Widget*)MItem);
		}
    }else {
        //app->messageBox("TODO : %s", gb->label());
        app->webViewWindow()->take_focus(); //focus on no meaning component,so give focus to webwindow
    }
}

