#include "Fl_MessageForm.h"

#include <FL/Fl_Box.H>
#include "Fl_GraphicButton.h"
#include "Fl_ImageCache.h"
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

Fl_Message_Form::Fl_Message_Form(int x, int y, int w, int h, const char* label)
    : Fl_Window(256, 205, 453, 131, label)
      , mMessage(0)
      , mIcon(0)
      , mYes(0)
      , mNo(0)
      , mBgImage(0)
      , mBangImage(0)
      , mIntegralImage(0)
{
    set_modal();
    clear_border();
    
    mBgImage = Fl_ImageCache::getInstance()->get("dialog-bg.png");
    mBangImage = Fl_ImageCache::getInstance()->get("Bang.png");    
    mIntegralImage = Fl_ImageCache::getInstance()->get("Integral.png");   

    mMessage = new Fl_Box(100, 10, 300, 70);
    mMessage->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_WRAP);
    mMessage->box(FL_FLAT_BOX);
    mMessage->color(fl_rgb_color(255, 251, 255));

    mIcon = new Fl_Box(60, 30, mBangImage->w(), mBangImage->h());   
    mIcon->align(FL_ALIGN_CENTER | FL_ALIGN_INSIDE);
    mIcon->box(FL_FLAT_BOX);
    mIcon->color(fl_rgb_color(255, 251, 255));

    mYes = new Fl_GraphicButton(100, 90, 53, 22, 0, "btn.png", "btn-hig.png");
    mYes->color(fl_rgb_color(255, 251, 255));
    mNo = new Fl_GraphicButton(300, 90, 53, 22, 0, "btn.png", "btn-hig.png");
    mNo->color(fl_rgb_color(255, 251, 255));
    mOK = new Fl_GraphicButton(200, 90, 53, 22, 0, "btn.png", "btn-hig.png");
    mOK->color(fl_rgb_color(255, 251, 255));

    end();
}

Fl_Message_Form::~Fl_Message_Form()
{
    if (mBgImage)
        mBgImage->release();
    if (mBangImage)
        mBangImage->release();
    if (mIntegralImage)
        mIntegralImage->release();
        
    if(mMessage != NULL) delete mMessage;
    if(mIcon != NULL) delete mIcon;
    if(mYes != NULL) delete mYes;
    if(mNo != NULL) delete mNo;
    if(mOK != NULL) delete mOK;	
}

void Fl_Message_Form::draw()
{
    if (damage() & ~FL_DAMAGE_CHILD) {
        //draw_box(box(), 0, 0, w(), h(), color()); // draw box with x/y = 0        
        if (mBgImage) {
            mBgImage->draw(0, 0);
        }
    }

    draw_children();
}

void Fl_Message_Form::message(const char* fmt, va_list ap)
{
    innards(fmt, ap, 0, 0, "Close");
}

void Fl_Message_Form::message(const char* fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    message(fmt, ap);
    va_end(ap);
}

bool Fl_Message_Form::ask(const char* fmt, va_list ap)
{    
    return (0 == innards(fmt, ap, "Yes", "No", NULL));
}

bool Fl_Message_Form::ask(const char* fmt, ...)
{
    bool r;
    va_list ap;

    va_start(ap, fmt);
    r = ask(fmt, ap);
    va_end(ap);
    return r;
}

int Fl_Message_Form::innards(const char* fmt, va_list ap, const char* yes, const char* no, const char* ok)
{
    vsnprintf(mBuffer, sizeof(mBuffer), fmt, ap);
    mMessage->label(mBuffer);
    if (ok) {
        mOK->label(ok);
        mOK->show();
        mYes->hide();
        mNo->hide();
        mIcon->image(mBangImage);
        mOK->take_focus();
    } else {
        mYes->label(yes);
        mYes->show();
        mNo->label(no);
        mNo->show();
        mOK->hide();
        mIcon->image(mIntegralImage);
        mYes->take_focus();
    }
    this->show();

    // deactivate Fl::grab(), because it is incompatible with Fl::readqueue()
    Fl_Window* g = Fl::grab();
    if (g) // do an alternative grab to avoid floating menus, if possible
        Fl::grab(this);

    int r;
    for (;;) {
        Fl_Widget* o = Fl::readqueue();
        if (!o) Fl::wait();
        else if (o == mYes) {r = 0; break;}
        else if (o == mNo) {r = 1; break;}
        else if (o == mOK) {r = 2; break;}
    }
    if (g) // regrab the previous popup menu, if there was one
        Fl::grab(g);
    this->hide();
    
    return r;
}


