#ifndef __Fl_SOFTKEYBOARD_YOUTUBE_EN_H__
#define __Fl_SOFTKEYBOARD_YOUTUBE_EN_H__

#include "Fl_SoftKeyboard.h"
#include "Fl_InputKB.h"

//Some Macro defines for YoutubeSoftKeyBoard
#define FL_INVALID_KEY       -1
#define FL_CHARACTERSET_KEY  0x1000
#define FL_COMP_KEY          0x1001
#define FL_AllCLEAR_KEY      0x1002
#define FL_RETURN_KEY        0x1003

#include <string>
using std::string;

class Fl_Shared_Image;

class Fl_SoftKeyboard_Youtube_EN : public Fl_SoftKeyboard{  
private:
    enum {
        NormalMode,
        ShiftMode,
        NumberMode,
        SymbolMode
    };

    static const int SoftKeyboardWidth = 545; 
    static const int SoftKeyboardHeight = 364;

    static const int SCANCODES = 41; //number of key

    static const int NO_FOCUS = -1;

    static const int NormalMap[SCANCODES];
    static const int ShiftMap[SCANCODES];
    static const int NumberMap[SCANCODES];
    static const int SymbolMap[SCANCODES];

    struct LayoutRect {
        int x, y, w, h;          
    };

    static const LayoutRect NormalLayoutRect[SCANCODES];
    static const LayoutRect ShiftLayoutRect[SCANCODES];
    static const LayoutRect NumberLayoutRect[SCANCODES];
    static const LayoutRect SymbolLayoutRect[SCANCODES];
    
    struct LayoutState {
        const char* mImageFile;
        const int* mMap;
        const LayoutRect* mLayoutRect;
    };

    static const LayoutState LayoutStates[4];

protected:
	virtual int handle(int);

	virtual void draw();
			
public:
    Fl_SoftKeyboard_Youtube_EN(int x, int y, int w, int h, const char* title=0);
    ~Fl_SoftKeyboard_Youtube_EN();
    
    /**
     *  @initValue, the initial text
     *  @x, the x position, will using old position when -1
     *  @y, the y position, will using old position when -1
     *  @return : return the text inputed by user, null means user cancel input.
     */
    const char* showSoftKeyboard(const char* initText = NULL, const bool isSecret = false, const int x = -1, const int y = -1);
    
    void showSoftKeyboard(const int x = -1, const int y = -1);
    /**
     * hide the soft keyboard, will make show return null.
     */
    void hideSoftKeyboard();

    int focus() { return mFocus; }
    void setFocus(int focus) {
        if (focus >= NO_FOCUS && focus < SCANCODES && mFocus != focus) {
            mOldFocus = mFocus;
            mFocus = focus;
            if (mLayout) {
                if (mOldFocus > NO_FOCUS && mOldFocus < SCANCODES) {
                    const LayoutRect& rect = mLayout->mLayoutRect[mOldFocus];
                    damage(FL_DAMAGE_ALL, rect.x, rect.y, rect.w, rect.h);
                }
                if (mFocus > NO_FOCUS && mFocus < SCANCODES) {
                    const LayoutRect& rect = mLayout->mLayoutRect[mFocus];
                    damage(FL_DAMAGE_ALL, rect.x, rect.y, rect.w, rect.h);
                }
            }
        }
    }

    void changeMode(int mode);
private:
    int mouseAt(int x, int y);
    int nextLine(int focus);
    int previousLine(int focus);
    int moveRight(int focus);
    int moveLeft(int focus);
    void processSoftKeyInput(int scancode);
    void processWebWiewInput(int scancode);

    int getKeyNumberByName(const int& num);

private:
    int mMode;
    int mOldFocus;
    const LayoutState* mLayout;
    Fl_Shared_Image* mLayoutImage;
    
    Fl_Group* mGroup;
    
    int mFocus;
    Fl_InputKB* mInput;
    bool mInterrupted;
};

#endif  //__Fl_SOFTKEYBOARD_YOUTUBE_EN_H__

