
#include <stdio.h>
#include <string.h>

#include <WebKit.h>
#include <directfb.h>

#include "Fl_SoftKeyboard_Youtube_EU.h"
#include "Fl_ImageCache.h"
#include "MainApplication.h"
#include "MainWebViewDelegate.h"
#include "Fl_WebViewWindow.h"

#include <FL/fl_draw.H>
#include <FL/x.H>

const int Fl_SoftKeyboard_Youtube_EU::NormalMap[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            FL_CHARACTERSET_KEY,
    'a', 'b', 'c', 'd', 'e', 'f',  FL_COMP_KEY,
    'g', 'h', 'i', 'j', 'k', 'l', 
    'm', 'n', 'o', 'p', 'q', 'r', FL_AllCLEAR_KEY,
    's', 't', 'u', 'v', 'w', 'x', 
    FL_RETURN_KEY, 'y', 'z', '.', '/', ':', '@', FL_BackSpace,
    '-', '_', ',', ' ', FL_Left, FL_Right
};

const int Fl_SoftKeyboard_Youtube_EU::LatinMap[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            FL_CHARACTERSET_KEY,	
    0x00e1, 0x00e0, 0x00e2, 0x00e4, 0x00e3, 0x00e7,  FL_COMP_KEY,
    0x00e9, 0x00e8, 0x00ea, 0x00eb, 0x00ed, 0x00ec, 
    0x00ee, 0x00ef, 0x00f1, 0x00f3, 0x00f2, 0x00f4, FL_AllCLEAR_KEY,
    0x00f6, 0x00f5, 0x00df, 0x00fa, 0x00f9, 0x00fb, 
    FL_RETURN_KEY,  0x00fc, 0x00ff, 0x00fd, FL_Left, FL_Right, FL_BackSpace,
     
};


const int Fl_SoftKeyboard_Youtube_EU::NumberMap[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            FL_CHARACTERSET_KEY,
    '1', '2', '3', '4', '5', '6',  FL_COMP_KEY,
    '7', '8', '9', '0', '.', '/', 
    ':', '@', '-', '_', ',', ' ', FL_AllCLEAR_KEY,
    FL_Left, FL_Right, 
    FL_RETURN_KEY, FL_BackSpace
};

const int Fl_SoftKeyboard_Youtube_EU::SymbolMap[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            FL_CHARACTERSET_KEY,
    '!' , '"', '#', '$', '%', '&',  FL_COMP_KEY,
    '\'', '*', '+', ',', '-', '.', 
    '/' , ':', ';', '<', '=', '>', FL_AllCLEAR_KEY,
    '?' , '@', '\\', '^', '_', '`', 
    FL_RETURN_KEY, '|', '~', '(', ')', '[', ']', FL_BackSpace,
    '{', '}', FL_Left, FL_Right
};

const Fl_SoftKeyboard_Youtube_EU::LayoutRect Fl_SoftKeyboard_Youtube_EU::NormalLayoutRect[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            			{73,76,400,30},	
    /*{22, 122, 103, 61},*/ {131, 122, 50, 30}, {182, 122, 50, 30}, {233, 122, 50, 30}, {284, 122, 50, 30}, {335, 122, 50, 30}, {386, 122, 50, 30}, {442, 122, 82, 61},
                        {131, 153, 50, 30}, {182, 153, 50, 30}, {233, 153, 50, 30}, {284, 153, 50, 30}, {335, 153, 50, 30}, {386, 153, 50, 30}, 
    /*{22, 184, 103, 61},*/ {131, 184, 50, 30}, {182, 184, 50, 30}, {233, 184, 50, 30}, {284, 184, 50, 30}, {335, 184, 50, 30}, {386, 184, 50, 30}, {442, 184, 82, 61},
    				    {131, 215, 50, 30}, {182, 215, 50, 30}, {233, 215, 50, 30}, {284, 215, 50, 30}, {335, 215, 50, 30}, {386, 215, 50, 30}, 
    {22, 246, 103, 61}, {131, 246, 50, 30}, {182, 246, 50, 30}, {233, 246, 50, 30}, {284, 246, 50, 30}, {335, 246, 50, 30}, {386, 246, 50, 30}, {442, 246, 82, 61},
                        {131, 277, 50, 30}, {182, 277, 50, 30}, {233, 277, 50, 30}, {284, 277, 50, 30}, {335, 277, 50, 30}, {386, 277, 50, 30}
};

const Fl_SoftKeyboard_Youtube_EU::LayoutRect Fl_SoftKeyboard_Youtube_EU::LatinLayoutRect[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            			{73,76,400,30},	
    /*{22, 122, 103, 61},*/ {131, 122, 50, 30}, {182, 122, 50, 30}, {233, 122, 50, 30}, {284, 122, 50, 30}, {335, 122, 50, 30}, {386, 122, 50, 30}, {442, 122, 82, 61},
                        {131, 153, 50, 30}, {182, 153, 50, 30}, {233, 153, 50, 30}, {284, 153, 50, 30}, {335, 153, 50, 30}, {386, 153, 50, 30}, 
    /*{22, 184, 103, 61},*/ {131, 184, 50, 30}, {182, 184, 50, 30}, {233, 184, 50, 30}, {284, 184, 50, 30}, {335, 184, 50, 30}, {386, 184, 50, 30}, {442, 184, 82, 61},
    				    {131, 215, 50, 30}, {182, 215, 50, 30}, {233, 215, 50, 30}, {284, 215, 50, 30}, {335, 215, 50, 30}, {386, 215, 50, 30}, 
    {22, 246, 103, 61}, {131, 246, 50, 30}, {182, 246, 50, 30}, {233, 246, 50, 30}, {284, 246, 50, 30}, {335, 246, 50, 30}, /*{386, 246, 50, 30}, */{442, 246, 82, 61},
                        /*{131, 277, 50, 30}, {182, 277, 50, 30}, {233, 277, 50, 30}, {284, 277, 50, 30}, {335, 277, 50, 30}, {386, 277, 50, 30}*/
};

const Fl_SoftKeyboard_Youtube_EU::LayoutRect Fl_SoftKeyboard_Youtube_EU::NumberLayoutRect[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            			{73,76,400,30},	
    /*{22, 122, 103, 61},*/ {131, 122, 50, 30}, {182, 122, 50, 30}, {233, 122, 50, 30}, {284, 122, 50, 30}, {335, 122, 50, 30}, {386, 122, 50, 30}, {442, 122, 82, 61},
                        {131, 153, 50, 30}, {182, 153, 50, 30}, {233, 153, 50, 30}, {284, 153, 50, 30}, {335, 153, 50, 30}, {386, 153, 50, 30}, 
    /*{22, 184, 103, 61},*/ {131, 184, 50, 30}, {182, 184, 50, 30}, {233, 184, 50, 30}, {284, 184, 50, 30}, {335, 184, 50, 30}, {386, 184, 50, 30}, {442, 184, 82, 61},
    				    {131, 215, 50, 30}, {182, 215, 50, 30}, /*{233, 215, 50, 30}, {284, 215, 50, 30}, {335, 215, 50, 30}, {386, 215, 50, 30},*/ 
    {22, 246, 103, 61}, /*{131, 246, 50, 30}, {182, 246, 50, 30}, {233, 246, 50, 30}, {284, 246, 50, 30}, {335, 246, 50, 30}, {386, 246, 50, 30},*/ {442, 246, 82, 61}
                        /*{131, 277, 50, 30}, {182, 277, 50, 30}, {233, 277, 50, 30}, {284, 277, 50, 30}, {335, 277, 50, 30}, {386, 277, 50, 30},*/ 
};

const Fl_SoftKeyboard_Youtube_EU::LayoutRect Fl_SoftKeyboard_Youtube_EU::SymbolLayoutRect[Fl_SoftKeyboard_Youtube_EU::SCANCODES] = {
	            			{73,76,400,30},	
    /*{22, 122, 103, 61},*/ {131, 122, 50, 30}, {182, 122, 50, 30}, {233, 122, 50, 30}, {284, 122, 50, 30}, {335, 122, 50, 30}, {386, 122, 50, 30}, {442, 122, 82, 61},
                        {131, 153, 50, 30}, {182, 153, 50, 30}, {233, 153, 50, 30}, {284, 153, 50, 30}, {335, 153, 50, 30}, {386, 153, 50, 30}, 
    /*{22, 184, 103, 61},*/ {131, 184, 50, 30}, {182, 184, 50, 30}, {233, 184, 50, 30}, {284, 184, 50, 30}, {335, 184, 50, 30}, {386, 184, 50, 30},  {442, 184, 82, 61},
    				    {131, 215, 50, 30}, {182, 215, 50, 30}, {233, 215, 50, 30}, {284, 215, 50, 30}, {335, 215, 50, 30}, {386, 215, 50, 30},
    {22, 246, 103, 61}, {131, 246, 50, 30}, {182, 246, 50, 30}, {233, 246, 50, 30}, {284, 246, 50, 30}, {335, 246, 50, 30}, {386, 246, 50, 30}, {442, 246, 82, 61},
                        {131, 277, 50, 30}, {182, 277, 50, 30}, {233, 277, 50, 30}, {284, 277, 50, 30} /*{335, 277, 50, 30}, {386, 277, 50, 30},*/ 
};

const Fl_SoftKeyboard_Youtube_EU::LayoutState Fl_SoftKeyboard_Youtube_EU::LayoutStates[4] = {
    {"im-eu/alphabet1_keyboard.png", NormalMap, NormalLayoutRect},
    {"im-eu/latin_keyboard.png",  LatinMap,  LatinLayoutRect},
    {"im-eu/number_keyboard.png", NumberMap, NumberLayoutRect},
    {"im-eu/sign_keyboard.png", SymbolMap, SymbolLayoutRect}
};

Fl_SoftKeyboard_Youtube_EU::Fl_SoftKeyboard_Youtube_EU(int x, int y, int w, int h, const char* title)
	: Fl_SoftKeyboard(x, y, SoftKeyboardWidth, SoftKeyboardHeight, "SoftKeyboard_YouTube_EN")
    , mMode(NormalMode)
    , mLayout(0)
    , mLayoutImage(0)
    , mGroup(0)    
    , mFocus(0)    
    , mInput(0)
    , mInterrupted(false)    
{
	mGroup = new Fl_Group(20,20,505,41);
	//mGroup->align(Fl_Align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE));
    mInput = new Fl_InputKB(20, 20, 505, 41);
    mInput->box(FL_UP_BOX);
    mInput->textsize(20);
    mGroup->end();
    
    hide();
    clear_border();
    set_modal();
    changeMode(NormalMode);        
    end();
}

Fl_SoftKeyboard_Youtube_EU::~Fl_SoftKeyboard_Youtube_EU()
{
    if (mLayoutImage != NULL)
        mLayoutImage->release();
    	  
    if(mGroup != NULL)
    	delete mGroup;
}

void Fl_SoftKeyboard_Youtube_EU::showSoftKeyboard(const int x, const int y)
{
	if (visible_r())
        return ;
    
    mInterrupted = false;

    if (x != -1 || y != -1)
        this->position(x == -1 ? this->x() : x, y == -1 ? this->y() : y);
    
    //mGroup->hide();   mark this for Youtube
    
    this->show(); 
}

const char* Fl_SoftKeyboard_Youtube_EU::showSoftKeyboard(const char* initText, const bool isSecret, const int x, const int y)
{
    if (visible_r())
        return NULL;

    mInterrupted = false;

    if (x != -1 || y != -1)
        this->position(x == -1 ? this->x() : x, y == -1 ? this->y() : y);
    if (isSecret)
        mInput->input_type(FL_INPUT_READONLY | FL_SECRET_INPUT);
    else
        mInput->input_type(FL_INPUT_READONLY);   
    
    if (initText) {
    	mInput->value(initText);
    	mInput->position(strlen(initText));
    }
    else
    	mInput->value("");    	
    	
    mInput->box(box());
    mGroup->show();
    
    this->show();
    Fl_Window* g = Fl::grab();
    if (g)
        Fl::grab(this);

    while(visible_r() && !mInterrupted && !fl_application_exited()) {
        Fl::wait();
    }

    if (g)
        Fl::grab(g);

    this->hide();
    
    if (mInterrupted || fl_application_exited()){
        return NULL;
    }

    return (mInput->value());
}

void Fl_SoftKeyboard_Youtube_EU::hideSoftKeyboard()
{
    if (!visible_r())
        return;

    mInterrupted = true;
}

void Fl_SoftKeyboard_Youtube_EU::draw()
{
    if (damage() & ~FL_DAMAGE_CHILD) {
        //draw_box(box(), 0, 0, w(), h(), color()); // draw box with x/y = 0	
        if (mLayoutImage) {
            mLayoutImage->draw(0, 0);
        }

        if (mFocus != NO_FOCUS ) {        
            const LayoutRect& focusRect = mLayout->mLayoutRect[mFocus];
            fl_color(0,255,255); //Set Focus color
            fl_rect(focusRect.x, focusRect.y, focusRect.w, focusRect.h);
        }
    }

    draw_children();
}

int Fl_SoftKeyboard_Youtube_EU::handle(int event)
{
	switch (event){
        case FL_KEYDOWN:
        //case FL_SHORTCUT:
            {
                int key = Fl::event_key();
                
                if (key == FL_Enter && mFocus != NO_FOCUS) {
                    int scancode = mLayout->mMap[mFocus];
                    if(mGroup->visible())
                    	processSoftKeyInput(scancode);
                    else
                    	processWebWiewInput(scancode);	
                } else if (key == FL_Left) {
                	if(mFocus==0){
                		mMode = (mMode - 1) < 0 ? (mMode - 1 + 4) : (mMode - 1) ;
                		changeMode(mMode);
                	}else{
                		setFocus(moveLeft(mFocus));                	
                	}
                } else if (key == FL_Right) {
                	if(mFocus==0){
                		mMode = (mMode + 1) % 4;
                		changeMode(mMode);
                	}else{
                		setFocus(moveRight(mFocus)); 
                	}
                } else if (key == FL_Down) {
                    setFocus(nextLine(mFocus));
                } else if (key == FL_Up) {
                    setFocus(previousLine(mFocus));
                } else if (key == FL_Home) {
                	this->hide();            
                } else {
                    return Fl_Window::handle(event);
                }
                return 1;
            }
        case FL_PUSH:        
        case FL_MOVE:
            {
                int focus = mouseAt(Fl::event_x(), Fl::event_y());
                setFocus(focus);
                if (mFocus != NO_FOCUS && event == FL_PUSH) {
                    int scancode = mLayout->mMap[mFocus];
                    if(mGroup->visible())
                    	processSoftKeyInput(scancode);
                    else
                    	processWebWiewInput(scancode);	
                }
                return 1;
            } 
		default:
			return Fl_Window::handle(event);	
	}
}

void Fl_SoftKeyboard_Youtube_EU::changeMode(int mode)
{
    mMode = mode;
    mLayout = &LayoutStates[mode];
    if (mLayoutImage)
        mLayoutImage->release();
    
    mLayoutImage = Fl_ImageCache::getInstance()->get(mLayout->mImageFile);
    mFocus = 0;

    redraw();
}

int Fl_SoftKeyboard_Youtube_EU::mouseAt(int x, int y)
{
    if (!mLayout)
        return NO_FOCUS;

    for(int i = 0; i <SCANCODES; i++) {
        const LayoutRect& rect = mLayout->mLayoutRect[i];
        if (x >= rect.x 
            && x < (rect.x + rect.w)
            && y >= rect.y
            && y < (rect.y + rect.h))
            return i;
    }
    return NO_FOCUS;
}

int Fl_SoftKeyboard_Youtube_EU::nextLine(int focus)
{
    if (focus == NO_FOCUS)
        return NO_FOCUS;
        
    if(mFocus == 0)
    	return 1; //Choose next focus for CharacterSet 

    const LayoutRect& focusRect = mLayout->mLayoutRect[focus];
    const int centerX = focusRect.x + focusRect.w / 2;
    int f = mouseAt(centerX, focusRect.y + focusRect.h + focusRect.h / 2);
    if (f == NO_FOCUS){
    	if(FL_RETURN_KEY == mLayout->mMap[focus]){
    		return focus;
    	}
    	else if(FL_BackSpace == mLayout->mMap[focus]){
    		return getKeyNumberByName(FL_COMP_KEY);
    	}
    	// choose next focus is CharacterSet.
    	return 0;
    	//return focus;
    }
    return f;
}

int Fl_SoftKeyboard_Youtube_EU::previousLine(int focus)
{ 
    if (focus == NO_FOCUS)
        return NO_FOCUS;

    if(FL_COMP_KEY == mLayout->mMap[focus]){
    	return getKeyNumberByName(FL_BackSpace);
    }
    else if(FL_CHARACTERSET_KEY == mLayout->mMap[focus]){
    	if(mMode == NormalMode)
    		return getKeyNumberByName('-');
    	else if(mMode == LatinMode)
    		return getKeyNumberByName(0x00fc);
    	else if(mMode == NumberMode)
    		return getKeyNumberByName(FL_Left);
    	else if(mMode == SymbolMode)
    		return getKeyNumberByName('{');
    	else
    		return -1;
    }

    const LayoutRect& focusRect = mLayout->mLayoutRect[focus];
    
    //Multi Lookup
    int f = NO_FOCUS;
    for(int i = 0; i < 2 ; i++)
    {
    	const int centerX = focusRect.x + (focusRect.w / 2);
    	f = mouseAt(centerX, focusRect.y - focusRect.h / 2  * (i + 1));
    	if(f != NO_FOCUS)
    		return f;
    }     
    return focus;
}

int Fl_SoftKeyboard_Youtube_EU::moveRight(int focus)
{ 
    if (focus == NO_FOCUS)
        return NO_FOCUS;
    
    // goto return key.
    if(mLayout->mMap[focus] == FL_BackSpace || mLayout->mMap[focus] == FL_COMP_KEY
    	|| mLayout->mMap[focus] == FL_AllCLEAR_KEY){
	    return getKeyNumberByName(FL_RETURN_KEY);
    }
    else if(FL_RETURN_KEY == mLayout->mMap[focus]){
        if(mMode==NumberMode){
            return 1;
        }
        return focus+1;	
    }
    else if(focus == 13){
    	return getKeyNumberByName(FL_COMP_KEY);
    }
    else if(focus == 26){
    	return getKeyNumberByName(FL_AllCLEAR_KEY);
    }
    else if(focus == 40){
    	return getKeyNumberByName(FL_BackSpace);
    }
    else if(mMode == NumberMode && FL_Right == mLayout->mMap[focus]){
    	return getKeyNumberByName(FL_AllCLEAR_KEY);
    }
    else if(mMode == SymbolMode && FL_Right == mLayout->mMap[focus]){
    	return getKeyNumberByName(FL_BackSpace);
    }
    
    
    return focus + 1;
}

int Fl_SoftKeyboard_Youtube_EU::moveLeft(int focus)
{ 
    if (focus == NO_FOCUS)
        return NO_FOCUS;

    // go to return key if current key is one of the leftest key of main key.
    if(focus == 1 || focus == 8 || focus == 14 || focus == 21 || focus == 35){
    	return getKeyNumberByName(FL_RETURN_KEY);
    }

    if(FL_RETURN_KEY == mLayout->mMap[focus]){
    	return getKeyNumberByName(FL_BackSpace);
    }
    	
    return focus - 1;	
}

void Fl_SoftKeyboard_Youtube_EU::processSoftKeyInput(int scancode)
{
    switch(scancode) {
		case FL_CHARACTERSET_KEY:
			mMode = (mMode + 1) % 4;
			changeMode(mMode);
			break;
        case FL_BackSpace:
            mInput->cutAsUTF8(-1);
			break;
        case FL_Left:
			mInput->moveCursorAsUTF8(-1);
            break;
        case FL_Right:
			mInput->moveCursorAsUTF8(1);
            break;
        case FL_COMP_KEY:
        	this->hide();
        	break;
        case FL_AllCLEAR_KEY:
        	mInput->replace(0,mInput->size(),"");
        	break;
        case FL_RETURN_KEY:
        	hideSoftKeyboard();
        	break;    
        default:
            {
				char out[5];
				if(convertToUtf8(scancode, out) < 0){
					return;
				}
				mInput->insert(out, strlen(out));                
            }
    }
}

int Fl_SoftKeyboard_Youtube_EU::convertToUtf8(int value, char * dest )
{
	if( value <= 0x7f ){
		*dest = value;
		*(dest+1) = 0;
		return 1;
	}   
	else if( value <= 0x7ff ){
		*dest = ( value >> 6 ) | 0xC0;
		*(dest+1) = ( value & 0x3F ) | 0x80;
		*(dest+2) = 0;
		return 2;
	}   
	else if( value <= 0xffff ){
		*dest = ( value >> 12 ) | 0xE0;
		*(dest+1) = ( ( value >> 6 ) & 0x3F ) | 0x80;
		*(dest+2) = ( value & 0x3F ) | 0x80;
		*(dest+3) = 0;
		return 3;
	}   
	else {
		printf("convert to utf8 error .\n");
		return -1; 
	}   
}

void Fl_SoftKeyboard_Youtube_EU::processWebWiewInput(int scancode)
{  
    DFBWindowEvent dfbevent;
    memset(&dfbevent,0,sizeof(DFBWindowEvent));
    dfbevent.clazz = DFEC_WINDOW;
    dfbevent.type  = DWET_KEYDOWN;
      
	MainApplication* app = MainApplication::application();
    WebView* webView = app->webViewWindow()->webView();
	
    switch(scancode) {
        case FL_BackSpace:
        	{
        		dfbevent.key_symbol = DIKS_BACKSPACE;
            	webView->onKeyDown((const DFBEvent&)dfbevent);
            }
            break;
        case FL_COMP_KEY:
        	this->hide();
        	break;
        case FL_AllCLEAR_KEY:
        	mInput->replace(0,mInput->size(),"");
        	break;
        case FL_RETURN_KEY:
        	hideSoftKeyboard();
        	break;                               
        default:
            {
            	char t = (char)scancode;
                if (isprint(t)) {
        			dfbevent.key_symbol = (DFBInputDeviceKeySymbol)t;
            		webView->onKeyDown((const DFBEvent&)dfbevent);
            	}
            }
    }
}

int Fl_SoftKeyboard_Youtube_EU::getKeyNumberByName(const int& num)
{
    for(int i = 0; i < SCANCODES; i++) {
    	if(num == mLayout->mMap[i]){
    		return i;
    	}
    }
    return NO_FOCUS;
}

