#ifndef __FL_SOFTKEYBOARD_YOUTUBE_JP_H__
#define __FL_SOFTKEYBOARD_YOUTUBE_JP_H__

#include "Fl_SoftKeyboard.h"
#include "Fl_InputKB.h"

//Some Macro defines for YoutubeSoftKeyBoard
#define FL_INVALID_KEY       -1
#define FL_CHARACTERSET_KEY  0x1000
#define FL_COMP_KEY          0x1001
#define FL_AllCLEAR_KEY      0x1002
#define FL_RETURN_KEY        0x1003
#define FL_SWITCH_KEY		0x1004
#define FL_KANJI_SWITCH		0x1005
#define FL_CONFIRM			0x1006
#define FL_DAKUTEN			0x1007
#define FL_HANDAKUTEN		0x1008

#define MAX_HIRA_SIZE		30

#include <string>
#include <vector>
using std::string;
using std::vector;

class Fl_Shared_Image;

class Fl_SoftKeyboard_Youtube_JP : public Fl_SoftKeyboard {  
	private:
		enum {
			HiraganaMode,
			KataKanaMode,
			NormalMode,
			ShiftMode,
			NumberMode,
			SymbolMode
		};

		// status for hiragana.
		enum HIRA_STATUS{
			HIRA_STATUS_UNINPUT,
			HIRA_STATUS_UNCHANGED,
			HIRA_STATUS_UNDECIDED,
		};

		static const int SoftKeyboardWidth = 545; 
		static const int SoftKeyboardHeight = 364;

		static const int SCANCODES = 44; //number of key

		static const int NO_FOCUS = -1;

		static const int Hiragana1Map[SCANCODES];
		static const int Hiragana2Map[SCANCODES];
		static const int Katakana1Map[SCANCODES];
		static const int Katakana2Map[SCANCODES];
		static const int NormalMap[SCANCODES];
		static const int ShiftMap[SCANCODES];
		static const int NumberMap[SCANCODES];
		static const int SymbolMap[SCANCODES];

		struct LayoutRect {
			int x, y, w, h;          
		};

		static const LayoutRect KeyLayoutRect[SCANCODES];

		struct LayoutState {
			const char* mImageFile;
			const int* mMap;
		};

		static const LayoutState LayoutStates[8];

		struct DakutenTable {
			int none;		
			int dakuten;		// 
			int handakuten;	// 
		};
		static const int DAKUTEN_COUNT = 40;
		static const DakutenTable DakutenTransTable[DAKUTEN_COUNT];

		// decode
		enum ConvertType {
			CONVERT_EUC_TO_UTF8 = 1,
			CONVERT_UTF8_TO_EUC = 0,		
		};

	protected:
		virtual int handle(int);

		virtual void draw();

	public:
		Fl_SoftKeyboard_Youtube_JP(int x, int y, int w, int h, const char* title=0);
		~Fl_SoftKeyboard_Youtube_JP();

		/**
		 *  @initValue, the initial text
		 *  @x, the x position, will using old position when -1
		 *  @y, the y position, will using old position when -1
		 *  @return : return the text inputed by user, null means user cancel input.
		 */
		const char* showSoftKeyboard(const char* initText = NULL, const bool isSecret = false, const int x = -1, const int y = -1);

		void showSoftKeyboard(const int x = -1, const int y = -1);
		/**
		 * hide the soft keyboard, will make show return null.
		 */
		void hideSoftKeyboard();

		int focus() { return mFocus; }
		void setFocus(int focus) {
			if (focus >= NO_FOCUS && focus < SCANCODES && mFocus != focus) {
				mOldFocus = mFocus;
				mFocus = focus;
				if (mLayout) {
					if (mOldFocus > NO_FOCUS && mOldFocus < SCANCODES) {
						const LayoutRect& rect = KeyLayoutRect[mOldFocus];
						damage(FL_DAMAGE_ALL, rect.x, rect.y, rect.w, rect.h);
					}
					if (mFocus > NO_FOCUS && mFocus < SCANCODES) {
						const LayoutRect& rect = KeyLayoutRect[mFocus];
						damage(FL_DAMAGE_ALL, rect.x, rect.y, rect.w, rect.h);
					}
				}
			}
		}

		void changeMode(int n);
	private:
		int mouseAt(int x, int y);
		int nextLine(int focus);
		int previousLine(int focus);
		int moveRight(int focus);
		int moveLeft(int focus);
		void processSoftKeyInput(int scancode);
		void processWebWiewInput(int scancode);

		int convertToUtf8(int value, char * dest );
		void resetHira();
		void fixHira();
		int transDakuten(int type);

		const char * convert(ConvertType type, const char* src);
	private:
		int mMode;
		int mOldFocus;
		const LayoutState* mLayout;
		Fl_Shared_Image* mLayoutImage;

		Fl_Group* mGroup;

		int mFocus;
		Fl_InputKB* mInput;
		bool mInterrupted;
		int mBeSwitched;

		Fl_InputKB* mInputHira;
		Fl_InputKB* mInputKanji;
		int mOffset;
		int mHiraSize;
		HIRA_STATUS mHiraStatus;
		int mLastInputCharacter;
		int mTransStatus;

		// for decode()
		std::string mEncodeBuf;
};

#endif  //__FL_SOFTKEYBOARD_YOUTUBE_JP_H__

