
#include "Fl_WebViewWindow.h"

#include "MainWebViewDelegate.h"
#include "MainDownloadDelegate.h"
#include <FL/x.H>

using namespace WebCore;

Fl_WebViewWindow::Fl_WebViewWindow(int w, int h)
    : Fl_Window(w, h, "VA_WEBKIT Browser")
{ 
    border(0);

    DFBRectangle rect = {0, 0, w, h};
    mWebView = WebView::createInstance();
    mWebView->initWithFrame(rect, NULL, NULL);
    mWebView->setViewWindow(this);

    mWebViewDelegate = new MainWebViewDelegate(this);
    mWebView->setWebNotificationDelegate(mWebViewDelegate);
    mWebView->setWebViewDelegate(mWebViewDelegate);
    mWebView->setWebFrameLoadDelegate(mWebViewDelegate);
    mWebView->setJSActionDelegate(mWebViewDelegate);

    mMainDownloadDelegate = new MainDownloadDelegate();
    mWebView->setDownloadDelegate(mMainDownloadDelegate);
}

Fl_WebViewWindow::Fl_WebViewWindow(int x, int y, int w, int h)
    : Fl_Window(x, y, w, h, "VA_WEBKIT Browser")
{
    border(0);

    DFBRectangle rect = {0, 0, w, h};
    mWebView = WebView::createInstance();
    mWebView->initWithFrame(rect, NULL, NULL);
    mWebView->setViewWindow(this);

    mWebViewDelegate = new MainWebViewDelegate(this);
    mWebView->setWebNotificationDelegate(mWebViewDelegate);
    mWebView->setWebViewDelegate(mWebViewDelegate);
    mWebView->setWebFrameLoadDelegate(mWebViewDelegate);
    mWebView->setJSActionDelegate(mWebViewDelegate);

    mMainDownloadDelegate = new MainDownloadDelegate();
    mWebView->setDownloadDelegate(mMainDownloadDelegate);
}

Fl_WebViewWindow::~Fl_WebViewWindow()
{
    delete mWebView;
    delete mMainDownloadDelegate;
    delete mWebViewDelegate;
}

void Fl_WebViewWindow::draw()
{
    BalEventExpose event;

    //Fl_Window::draw();

    IDirectFBSurface* surface = reinterpret_cast<IDirectFBSurface*>(getClientSurface());

    /////////FIXME////////////////it voilate fltk
    if (surface) {
        DFBRectangle dirtyRect = mWebView->dirtyRegion();
        BalEventExpose event;

        event.dirty.x1 = dirtyRect.x;
        event.dirty.y1 = dirtyRect.y;
        event.dirty.x2 = dirtyRect.x + dirtyRect.w - 1;
        event.dirty.y2 = dirtyRect.y + dirtyRect.h - 1;

        surface->SetClip(surface, &event.dirty);
        mWebView->setViewWindow(this);
        mWebView->onExpose(event);
        surface->Flip(surface, &event.dirty, DSFLIP_BLIT);
    }
}

int Fl_WebViewWindow::handle(int event)
{
    if (!mWebView || !fl_dfb_event)
        return Fl_Window::handle(event);

    //printf("%s : event = %d\n", __FUNCTION__, event);

    DFBEvent dfbevent;
    
    switch(event) {
    	case FL_FOCUS:
        case FL_UNFOCUS:
            return 1;
        case FL_PUSH:
            if (fl_FLEventToDFBEvent(event, dfbevent))
            mWebView->onMouseButtonDown(dfbevent);
            return 1;
        case FL_RELEASE:
            if (fl_FLEventToDFBEvent(event, dfbevent))
            mWebView->onMouseButtonUp(dfbevent);
            return 1;
        case FL_MOVE:
        case FL_DRAG:
            if (fl_FLEventToDFBEvent(event, dfbevent))
            mWebView->onMouseMotion(dfbevent);
            return 1;

        case FL_SHORTCUT:
        case FL_KEYDOWN:
        	if (Fl::event_key() == FL_Home)
        		return 0;
            if (fl_FLEventToDFBEvent(event, dfbevent))
            	return mWebView->onKeyDown(dfbevent);
            return 1;
        case FL_KEYUP:
        	if (Fl::event_key() == FL_Home)
        		return 0;
            if (fl_FLEventToDFBEvent(event, dfbevent))
            	return mWebView->onKeyUp(dfbevent);
            return 1;

        default:
            return Fl_Window::handle(event);
    }
}

