#ifndef __MainApplication_H__
#define __MainApplication_H__


class MainUI;
class Fl_Box;
class Fl_Window;
class Fl_Group;
class Fl_SoftKeyboard;
class Fl_Message_Form;
class Fl_WebViewWindow;

class Fl_SoftKeyboard;
class Fl_MainMenu;
class Fl_Message_Form;

class Bookmark;
class History;
class WebPreferencesInterface;
class WebCookieJar;

class MainApplication {    
public:
    static MainApplication* application();

    MainApplication(int argc, char* argv[]);

    virtual ~MainApplication();

    virtual void onInit();

    virtual void onExit();

    virtual int run();

    void updateTitle(const char* title);
    
    void updateStatus(const char* status);

    bool messageBox(const char* fmt, ...);

    void messageInfo(const char* fmt, ...);

    int shortcut_handler(int event);

public:

    Fl_Window* mainWindow();
    Fl_Group* mainGroup();
    Fl_WebViewWindow* webViewWindow();
    Fl_SoftKeyboard* softKeyboard();
    Fl_Message_Form* messageForm();
    Fl_Box* titleBar();
    Fl_Box* statusBar();
    Fl_MainMenu* mainMenu();

    MainUI* mainUI() { return mMainUI; }

    WebCookieJar* cookieJar() { return mCookieJar; }
    WebPreferencesInterface* preferences() { return mPref; }
    Bookmark* bookmark() { return mBookmark; }
    History* history() { return mHistory; }

public:
	void test();
    void loadSettings();
    void saveSettings();
    
    void processSettingsData(char* root_content);
    void getSettingData(char** root_content);

private:
    MainUI* mMainUI;
    bool mBrowserMode;
    const char* mURL;
    
    //
    WebCookieJar* mCookieJar;
    WebPreferencesInterface* mPref;
    Bookmark* mBookmark;
    History* mHistory;
};

#endif  //__MainApplication_H__

