#include <FL/fl_ask.H>
#include <FL/Fl.H>

#include "MainWebViewDelegate.h"
#include "Fl_WebViewWindow.h"
#include "MainApplication.h"
#include "Fl_SoftKeyboard.h"

MainWebViewDelegate::MainWebViewDelegate(Fl_WebViewWindow* viewWindow)
    : mViewWindow(viewWindow)
    , mShowingSoftKeyboard(false)
    , mProgress(0)
{
    mTitle[0] = mSynthesizeTitle[0] = mStatus[0] = 0;
}

MainWebViewDelegate::~MainWebViewDelegate()
{

}

void MainWebViewDelegate::displaySoftKeyboard(WebView* view, const int x, const int y, const int w, const int h)
{    
    Fl_SoftKeyboard* softKeyboard = MainApplication::application()->softKeyboard();
    int scr_x, scr_y, scr_w, scr_h;
    int des_x, des_y;
    //Fl::screen_xywh(scr_x, scr_y, scr_w, scr_h);
    scr_x = MainApplication::application()->webViewWindow()->x();
    scr_y = MainApplication::application()->webViewWindow()->y();
    scr_w = MainApplication::application()->webViewWindow()->w();
    scr_h = MainApplication::application()->webViewWindow()->h();
    
    if((x + w/2 + softKeyboard->w()/2 <= scr_w) && (x + w/2 - softKeyboard->w()/2 >= 0))
    {
    	des_x = scr_x + x + w/2 - softKeyboard->w()/2;
    }else if(x + w/2 + softKeyboard->w()/2 > scr_w){
    	des_x = scr_x + scr_w - softKeyboard->w();
    }else{
    	des_x = scr_x;	
    }	
    
    if(y + h + 6 + softKeyboard->h() <= scr_h)
    {
    	des_y = scr_y + y + h + 6;
    }else{
    	des_y = scr_y + y - 6 - softKeyboard->h();	
    }	
    	
    if (softKeyboard)
    {
    	//softKeyboard->showSoftKeyboard(des_x, des_y);
    	softKeyboard->showSoftKeyboard(207, 88);
    }
}

const char* MainWebViewDelegate::displaySoftKeyboard(WebView* view, const char* initialValue, const bool isSecret, const int x, const int y)
{
    const char* value = NULL;

    if (mShowingSoftKeyboard)
        return NULL;

    mShowingSoftKeyboard = true; // TODO show softkeyboard window in here.
    
    Fl_SoftKeyboard* softKeyboard = MainApplication::application()->softKeyboard();
    if (softKeyboard)
    {
    	//value = softKeyboard->showSoftKeyboard(initialValue, isSecret, 145, 255);
    	value = softKeyboard->showSoftKeyboard(initialValue, isSecret, 207, 88);
    }
    mShowingSoftKeyboard = false;

    return value;
}

void MainWebViewDelegate::hideSoftKeyboard(WebView* view)
{
    if (mShowingSoftKeyboard) {
        //
        // mSoftKeyboard->hide();
        //
    }
}

void MainWebViewDelegate::hoverOverLink(WebView* view, const char* title, const char* link)
{
    //printf("%s : title = %s, link = %s\n", __FUNCTION__, title, link);
    if (title && title[0] && link)
        snprintf(mStatus, BUFFER_LENGTH, "%s -- %s", title, link);
    else if (link)
        snprintf(mStatus, BUFFER_LENGTH, "%s", link);
    else
        mStatus[0] = 0;
    MainApplication::application()->updateStatus(mStatus);
}

void MainWebViewDelegate::setStatusBarText(WebView* view, const char* status)
{
    //printf("%s : status = %s\n", __FUNCTION__, status);
    snprintf(mStatus, BUFFER_LENGTH, "%s", status);
    MainApplication::application()->updateStatus(mStatus);
}

bool MainWebViewDelegate::shouldInterruptJavaScript()
{
    return MainApplication::application()->messageBox("JavaScript seems hang, Do you want to interrupt it?");
}

void MainWebViewDelegate::updateMainUIStatus()
{
    if (mProgress < 100 && mProgress > 0)
        snprintf(mSynthesizeTitle, BUFFER_LENGTH, "%s ---- %02d%%", mTitle, mProgress);
    else
        snprintf(mSynthesizeTitle, BUFFER_LENGTH, "%s", mTitle);
    MainApplication::application()->updateTitle(mSynthesizeTitle);
}

void MainWebViewDelegate::titleChange(WebFrame* frame, const char* title)
{
    //printf("%s : %s\n", __FUNCTION__, title);
    if (frame && !frame->parentFrame()) {
        snprintf(mTitle, BUFFER_LENGTH, "%s", title);
        updateMainUIStatus();
    }
}

void MainWebViewDelegate::progressNotification(WebFrame* webFrame)
{
    mProgress = int(webFrame->webView()->estimatedProgress() * 100);
    printf("progress : %d \n", mProgress);
    updateMainUIStatus();
}

void MainWebViewDelegate::finishedLoadNotification(WebFrame*)
{
    if (mProgress != 100) {
        mProgress = 100;
        printf("progress : %d \n", mProgress);
        updateMainUIStatus();
    }
}

