#ifndef __MainWebViewDelegate_H__
#define __MainWebViewDelegate_H__

#include <WebKit.h>
#include <stdio.h>

class Fl_WebViewWindow;

class MainWebViewDelegate : public WebViewDelegate, public WebFrameLoadDelegate, public WebNotificationDelegate, public JSActionDelegate {

public:
    MainWebViewDelegate(Fl_WebViewWindow*);
    ~MainWebViewDelegate();

    ////  WebViewDelegate
    void displaySoftKeyboard(WebView* view, const int x = -1, const int y = -1, const int w = -1, const int h = -1);
    
    const char* displaySoftKeyboard(WebView* view, const char* initialValue, const bool isSecret = false, const int x = -1, const int y = -1);

    void hideSoftKeyboard(WebView* view);

    void hoverOverLink(WebView* view, const char* title, const char* link);

    void setStatusBarText(WebView* view, const char* status);

    bool shouldInterruptJavaScript();

    //// WebFrameLoadDelegate
    /**
     * windowObjectClearNotification
     */
    void windowObjectClearNotification(WebFrame*, void*, void*)
    {
    }

    /**
     * titleChange : call when the title change
     */
    void titleChange(WebFrame* frame, const char* title);

    /**
     * didStartLoad : called when a frame starts loading
     */
    void didStartProvisionalLoad(WebFrame*)
    {
    }

    /**
     * didReceiveServerRedirectForProvisionalLoadForFrame: called when a server-side redirection occurs.
     */
    void didReceiveServerRedirectForProvisionalLoadForFrame(WebFrame*)
    {
    }

    /**
     * willPerformClientRedirectToURL: called when a client-side will occur.
     */
    virtual void willPerformClientRedirectToURL(WebFrame*, const char* url, double delaySeconds, double fireDate)
    {
    }

    /**
     * didCancelClientRedirectForFrame: called when a client-side redirection was cancelled before it occurred.
     */
    virtual void didCancelClientRedirectForFrame(WebFrame*)
    {
    }

    /**
     * didCommitLoad : called when a frame data source transitions from provisional state to committed
     */
    virtual void didCommitLoad(WebFrame*)
    {
    }

    /**
     * didFinishLoad : called when a frame has finished loading itself and its resources
     */
    virtual void didFinishLoad(WebFrame*)
    {
    }

    /**
     * didFailLoad : called if a frame loading failed
     */
    // FIXME: We should use a WebError argument once it is exported to third party application.
    virtual void didFailLoad(WebFrame*)
    {
    }

    //// WebNotificationDelegate
    virtual void startLoadNotification(WebFrame*)
    {
    }

    virtual void progressNotification(WebFrame* webFrame);

    virtual void finishedLoadNotification(WebFrame*);

    //// JSActionDelegate
    //virtual void windowObjectClearNotification(WebFrame*, void*, void*)
    //{
    //}

    virtual void consoleMessage(WebFrame*, int line, const char *message)
    {
        printf("CONSOLE MESSAGE: line %d: %s\n", line, message);
    }

    virtual bool jsAlert(WebFrame *frame, const char *message)
    {
        //fl_alert(message);
        printf("JS ALERT: frame %p: %s\n", frame, message);        
        return true;
    }

    virtual bool jsConfirm(WebFrame *frame, const char *message)
    {
        //return fl_choice(message, "Yes", "No", 0L);
        printf("JS CONFIRM: frame %p: %s\n", frame, message);
        return false;
    }

    virtual bool jsPrompt(WebFrame *frame, const char *message, const char *defaultValue, char **value)
    {
        /*
         *  const char* input = fl_input(defaultValue, message);
         *  if (input) {
         *      strcpy(*value, input);
         *      return true;
         *  } else
         *      return false;
         */
        printf("JS PROMPT: frame %p: %s, defaultValue = %s\n", frame, message, defaultValue ? defaultValue : "null");
        return false;
    }

private:
    void updateMainUIStatus();

private:
    static const int BUFFER_LENGTH = 1024;

    Fl_WebViewWindow* mViewWindow;
    bool mShowingSoftKeyboard;
    char mTitle[BUFFER_LENGTH];
    int mProgress;
    char mSynthesizeTitle[BUFFER_LENGTH];
    char mStatus[BUFFER_LENGTH];
};

#endif  //__MainWebViewDelegate_H__

