/* ***** BEGIN LICENSE BLOCK *****
 * (C)opyright 2008 Aplix Corporation. anselm@aplixcorp.com
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * ***** END LICENSE BLOCK ***** */

#include <stdio.h>
#include <string.h>

#include "npapi.h"
#include "npfunctions.h"
#include "npruntime.h"
#define OSCALL
#define OSEXPORT __attribute__((visibility("default")))

enum {
    PLAY_CMD,
    PAUSE_CMD,
    RESUME_CMD,
    SEEK_CMD,
    STOP_CMD,
    STATUS_CMD
};

enum {
    STOP = 0,
    PLAY,
    PAUSE,
};

static int status = STOP;
static int startTime;
static NPObject *so              = NULL;
static NPNetscapeFuncs *npnfuncs = NULL;


/* NPN */

static void logmsg(const char *msg) {
	fputs(msg, stderr);
}

static bool
hasMethod(NPObject* obj, NPIdentifier methodName) {
	logmsg("npsimple: hasMethod\n");
	return true;
}

static bool
invokeDefault(NPObject *obj, const NPVariant *args, uint32_t argCount, NPVariant *result) {
	logmsg("npsimple: invokeDefault\n");
	result->type = NPVariantType_Int32;
	result->value.intValue = 42;
	return true;
}

static bool
invoke(NPObject* obj, NPIdentifier methodName, const NPVariant *args, uint32_t argCount, NPVariant *result) {
	logmsg("npsimple: invoke\n");
	char *name = npnfuncs->utf8fromidentifier(methodName);
    printf("npsimple: invoke %s\n", name);
	if(name && !strcmp((const char *)name, "foo")) {
		return invokeDefault(obj, args, argCount, result);
	} if (name && !strcasecmp((const char*)name, "request")) {
        int cmd = -1;
        for(int i = 0; i < argCount; i++) {
            const NPVariant arg = args[i];
            printf("npsimple: invoke args[%d] = ", i);
            switch(arg.type) {
                case NPVariantType_Void:
                    printf("void\n");
                    break;
                case NPVariantType_Null:
                    printf("null\n");
                    break;
                case NPVariantType_Bool:
                    printf("bool %s\n", arg.value.boolValue ? "true" : "false");
                    break;
                case NPVariantType_Int32:
                    printf("int %d\n", arg.value.intValue);
                    break;
                case NPVariantType_Double:
                    printf("double %lf\n", arg.value.doubleValue);
                    break;
                case NPVariantType_String:
                    printf("string %s\n", arg.value.stringValue.UTF8Characters);
                    if (strstr(arg.value.stringValue.UTF8Characters, "play"))
                        cmd = PLAY_CMD;
                    else if (strstr(arg.value.stringValue.UTF8Characters, "pause"))
                        cmd = PAUSE_CMD;
                    else if (strstr(arg.value.stringValue.UTF8Characters, "resume"))
                        cmd = RESUME_CMD;
                    else if (strstr(arg.value.stringValue.UTF8Characters, "seek"))
                        cmd = SEEK_CMD;
                    else if (strstr(arg.value.stringValue.UTF8Characters, "status"))
                        cmd = STATUS_CMD;
                    break;
                case NPVariantType_Object:
                    printf("object\n");
                    break;
                default:
                    printf("unknow\n");
                    break;
            }
        }
        result->type = NPVariantType_Int32;
        result->value.intValue = -1;
 //       return true;
        switch(cmd) {
            case PLAY_CMD:
                status = PLAY;
                result->value.intValue = 0;
                startTime = 0;
                break;
            case PAUSE_CMD:
                if (status == PLAY) {
                    status = PAUSE;
                    result->value.intValue = 0;
                } else 
                    result->value.intValue = 1;
                break;
            case RESUME_CMD:
                if (status == PAUSE) {
                    status = PLAY;
                    result->value.intValue = 0;
                } else
                    result->value.intValue = 1;
                break;
            case SEEK_CMD:
                result->value.intValue = 0;
                break;
            case STOP_CMD:
                if (status == PLAY || status == PAUSE) {
                    status = STOP;
                    result->value.intValue = 0;
                } else
                    result->value.intValue = 1;
                break;
            case STATUS_CMD:
                startTime++;
                if (startTime == 30)
                    status = STOP;               
                result->value.intValue = (status == PLAY) ? 5 : ((status == PAUSE) ? 4 : 2);
                break;
            default:
                result->value.intValue = -1;
        }
        return true;
    }
	else {
		// aim exception handling
		npnfuncs->setexception(obj, "exception during invocation");
		return false;
	}
}

static bool
hasProperty(NPObject *obj, NPIdentifier propertyName) {
	logmsg("npsimple: hasProperty\n");
	char *name = npnfuncs->utf8fromidentifier(propertyName);
    printf("npsimple: hasProperty %s\n", name);
	return false;
}

static bool
getProperty(NPObject *obj, NPIdentifier propertyName, NPVariant *result) {
	logmsg("npsimple: getProperty\n");
	char *name = npnfuncs->utf8fromidentifier(propertyName);
    printf("npsimple: hasProperty %s\n", name);
	return false;
}

static NPClass npcRefObject = {
	NP_CLASS_STRUCT_VERSION,
	NULL,
	NULL,
	NULL,
	hasMethod,
	invoke,
	invokeDefault,
	hasProperty,
	getProperty,
	NULL,
	NULL,
    NULL,
    NULL
};

/* NPP */

static NPError
nevv(NPMIMEType pluginType, NPP instance, uint16 mode, int16 argc, char *argn[], char *argv[], NPSavedData *saved) {
	logmsg("npsimple: new\n");
	return NPERR_NO_ERROR;
}

static NPError
destroy(NPP instance, NPSavedData **save) {
	if(so)
		npnfuncs->releaseobject(so);
	so = NULL;
	logmsg("npsimple: destroy\n");
	return NPERR_NO_ERROR;
}

static NPError
getValue(NPP instance, NPPVariable variable, void *value) {
	switch(variable) {
	default:
		logmsg("npsimple: getvalue - default\n");
		return NPERR_GENERIC_ERROR;
	case NPPVpluginNameString:
		logmsg("npsimple: getvalue - name string\n");
		*((char **)value) = "AplixFooPlugin";
		break;
	case NPPVpluginDescriptionString:
		logmsg("npsimple: getvalue - description string\n");
		*((char **)value) = "<a href=\"http://www.aplix.co.jp/\">AplixFooPlugin</a> plugin.";
		break;
	case NPPVpluginScriptableNPObject:
		logmsg("npsimple: getvalue - scriptable object\n");
		if(!so)
			so = npnfuncs->createobject(instance, &npcRefObject);
		npnfuncs->retainobject(so);
		*(NPObject **)value = so;
		break;
	}
	return NPERR_NO_ERROR;
}

static NPError /* expected by Safari on Darwin */
handleEvent(NPP instance, void *ev) {
	logmsg("npsimple: handleEvent\n");
	return NPERR_NO_ERROR;
}

static NPError /* expected by Opera */
setWindow(NPP instance, NPWindow* pNPWindow) {
	logmsg("npsimple: setWindow\n");
	return NPERR_NO_ERROR;
}

/* EXPORT */
#ifdef __cplusplus
extern "C" {
#endif

OSEXPORT NPError OSCALL
NP_GetEntryPoints(NPPluginFuncs *nppfuncs)
{
	logmsg("npsimple: NP_GetEntryPoints\n");
	nppfuncs->version       = (NP_VERSION_MAJOR << 8) | NP_VERSION_MINOR;
	nppfuncs->newp          = nevv;
	nppfuncs->destroy       = destroy;
	nppfuncs->getvalue      = getValue;
	nppfuncs->event         = handleEvent;
	nppfuncs->setwindow     = setWindow;

	return NPERR_NO_ERROR;
}

#ifndef HIBYTE
#define HIBYTE(x) ((((uint32)(x)) & 0xff00) >> 8)
#endif

OSEXPORT NPError OSCALL
NP_Initialize(NPNetscapeFuncs *npnf, NPPluginFuncs *nppfuncs) {
	logmsg("npsimple: NP_Initialize\n");
	if(npnf == NULL)
		return NPERR_INVALID_FUNCTABLE_ERROR;

	if(HIBYTE(npnf->version) > NP_VERSION_MAJOR)
		return NPERR_INCOMPATIBLE_VERSION_ERROR;

	npnfuncs = npnf;
	NP_GetEntryPoints(nppfuncs);
	return NPERR_NO_ERROR;
}

OSEXPORT NPError OSCALL 
NP_Shutdown() {
	logmsg("npsimple: NP_Shutdown\n");
	return NPERR_NO_ERROR;
}

OSEXPORT char * OSCALL
NP_GetMIMEDescription() {
	logmsg("npsimple: NP_GetMIMEDescription\n");
	return "application/x-inettv-vod-plugin:.foo:anselm@aplix.co.jp";
	//return "application/x-vnd-aplix-foo:.foo:anselm@aplix.co.jp";
}

OSEXPORT NPError OSCALL
/* needs to be present for WebKit based browsers */
NP_GetValue(void *npp, NPPVariable variable, void *value) {
	return getValue((NPP)npp, variable, value);
}

#ifdef __cplusplus
}
#endif
